-- $Id: aragorn.lua 2379 2016-05-01 23:30:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

newChat {
  id = 'welcome',
  text = [[Welcome and well met, @playername@.  Tales of your adventures precede you.]],
  answers = {
    { 'Thank you, Your Highness. [leave]' },
  },
}

newChat {
  id = 'narsil1',
  text = [[Welcome, @playername@.  My sources tell me you have brought something of great interest to show me.]],
  answers = {
    { [[I have indeed, Your Highness. #LIGHT_GREEN#*show Narsil to Aragorn*#LAST#]], jump = 'narsil2' },
  },
}

newChat {
  id = 'narsil2',
  text = [[...Remarkable.  This is truly Narsil, the sword of my ancestors, broken so many years ago.  Where did you find­ no, that is not important.  Here, let me take it to the royal swordsmiths.  The sword that was broken shall be reforged at last.]],
  answers = {
    {
      [[#LIGHT_GREEN#*give Narsil to Aragorn*#LAST#]],
      action = function(npc, player)
	local o, item, inven_id = player:findInAllInventoriesBy('define_as', 'NARSIL')
	if o then
	  -- Sneaky Hack(TM), part 1:  Keep a copy of Narsil's found{} info
	  -- to put on Andúril.
	  player.tmp_narsil_found = o.found:clone()
	  player:removeObject(inven_id, item, true)
	  -- We do NOT call o:removed() here, as that would put Narsil back
	  -- in circulation, which would just be silly.
	  if game and game.hasEntity and game:hasEntity(o) then game:removeEntity(o) end
	end
      end,
      jump = 'narsil3'
    },
  },
}

newChat {
  id = 'narsil3',
  text = [[#LIGHT_GREEN#*Aragorn returns a few hours later.*#LAST#  Behold, Andúril, Flame of the West, the Sword That Was Broken and is now reforged.  Here, I offer it into your keeping for now, @playername@; my heart tells me that you have a momentous fate ahead of you, and you will have need of a sturdy weapon.]],
  answers = {
    {
      [[Thank you, Your Highness. #LIGHT_GREEN#*take Andúril*#LAST#]],
      action = function(npc, player)
	local o = game.zone:makeEntityByName(game.level, 'artifact', 'ANDURIL')
	if o then
	  o:identify('full')
	  -- Sneaky Hack(TM), part 2:  Munge Andúril's found information to
	  -- try to remember where we found Narsil.
	  o.found = player.tmp_narsil_found or { type='missing-narsil-info' }
	  player.tmp_narsil_found = nil
	  local ff = o.found
	  local pfx = '%s was reforged from the shards of Narsil, which '
	  if ff.type == 'birth' then
	    ff.type = 'custom'
	    ff.custom = pfx .. 'you began the game with (how?).'
	  elseif ff.type == 'floor' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found on the '..(ff.town_zone and 'ground' or 'floor')
	  elseif ff.type == 'mon_drop' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found in the remains of '..ff.mon_name
	  elseif ff.type == 'vault' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found in a vault'
	  elseif ff.type == 'rubble' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found buried under rubble'
	  elseif ff.type == 'chest' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found in a chest'
	  elseif ff.type == 'store_buy' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you bought from the '..ff.store_name
	  elseif ff.type == 'debug_dialog' then
	    ff.type = 'custom'
	    ff.custom = pfx .. 'you created from the debug dialog.'
	  elseif ff.type == 'custom' or ff.type == 'custom_with_level' then
	    -- Questionable Hack(TM):  Assume the custom text is roughly of
	    -- the form 'You magically conjured %s into existence', which
	    -- we convert to 'you magically conjured into existence'.
	    ff.custom = pfx .. ff:custom:gsub('You', 'you', 1):gsub(' %%s', '', 1)
	  else
	    ff.custom = '??? ' .. pfx .. ("you found somewhere strange (type='%s')"):format(tostring(ff.type or '???'))
	    ff.type = 'custom'
	  end
	  player:addOrDropObject(player.INVEN_INVEN, o, nil, true)
	  player:sortInven()
	  game.logPlayer(player, 'You receive:  %s', o:getName())
	  player:setQuestStatus('narsil', engine.Quest.DONE)
	  world:gainAchievement('NARSIL', player)
	end
      end,
    },
  },
}

if game.player:findInAllInventoriesBy('define_as', 'NARSIL') then
  return 'narsil1'
end
return 'welcome'
