-- $Id: quest-relic-find.lua 1522 2015-07-23 18:19:04Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

local Birther = require('engine.Birther')

local god_d = Birther:getBirthDescriptor('god', game.player.descriptor.god)
-- Slight Hack(TM):  All the god descriptions begin with "He/She is...".
local title = god_d.desc:find('^She') and 'Milady' or 'Milord'

local q = game.player:hasQuest('relic')
local n_quests_done = q and q:num_done() or 0
local boost = (n_quests_done == 5 and 10 or 5) * game.player.skills.PRAYER.mod
local boost_fmt = ('%g'):format(boost/game.player.skill_step)

-- These only contain the quips for pieces 1 thru 4; the last piece's quips
-- are already defined in the entry5/reward5 entries.
local quips = {
  {
    '#{italic}#*blink*#{normal}# O-kaaay...',
    '#{italic}#*whew*#{normal}# Finally.',
    "The scary part is, I'm actually getting used to this.",
    "Y'know, it'd be #{italic}#so#{normal}# much easier if you could do this directly...",
  },
  {
    'Oh, that sounds delightful...',
    "So, hang on, you've been holding out on me prayer-wise all this time?",
    'The scarier part is, I was fully expecting that too.',
    'Uh-huh... wait, did you say "last"?',
  },
}

local function prayer_boost(npc, player)
  player:adjSkillValue('PRAYER', boost)
  if n_quests_done == 1 then
    world:gainAchievement('RELIC_PIECE', player)
  end
  if n_quests_done == 5 then
    player:setQuestStatus('relic', engine.Quest.DONE)
    local Birther = require 'engine.Birther'
    local god_d = Birther:getBirthDescriptor('god', player.descriptor.god)
    world:gainAchievement(god_d.quest.achievement, player)
  end
  return n_quests_done == 5 and 'reward5' or 'reward'
end

newChat {
  id = 'enter',
  text = [[#LIGHT_GREEN#@playerdescriptor.god@ speaks to you:#LAST#
Well done, @playername@!  Now that you have found the relic piece, I can take it into My safekeeping.  #LIGHT_GREEN#[the relic vanishes]#LAST#]],
  answers = {
    { 'All to your glory, '..title..'.', action=prayer_boost },
    { quips[1][n_quests_done], action=prayer_boost },
  },
}

newChat {
  id = 'reward',
  text = [[As a reward for your efforts, I shall teach you how to pray more effectively #LIGHT_BLUE#(+]]..boost_fmt..[[ Prayer skill)#LAST#.  And be vigilant!  For once ]]..(n_quests_done == 4 and 'the last' or 'another')..[[ of my temples is found, I shall surely call upon you again!]],
  answers = {
    { 'Thank you, '..title..'.' },
    { quips[2][n_quests_done] },
  },
}

newChat {
  id = 'enter5',
  text = [[#LIGHT_GREEN#@playerdescriptor.god@ speaks to you:#LAST#
At last!  My relic is complete once again!  #LIGHT_GREEN#[the relic vanishes]#LAST#]],
  answers = {
    { 'All to your glory, '..title..'.', action=prayer_boost },
    { "And this is it?  We're #{italic}#finally#{normal}# done?", action=prayer_boost },
  },
}

newChat {
  id = 'reward5',
  text = [[As a reward for your exceptional efforts, I shall teach you the most potent liturgies of prayer #LIGHT_BLUE#(+]]..boost_fmt..[[ Prayer skill)#LAST#.]],
  answers = {
    { 'Thank you, '..title..'.' },
    { "Halle-freakin'-lujah..." },
  },
}

return n_quests_done == 5 and 'enter5' or 'enter'
