-- $Id: quest-relic.lua 1367 2015-04-20 02:45:30Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

local Birther = require('engine.Birther')

local god_d = Birther:getBirthDescriptor('god', game.player.descriptor.god)
-- Slight Hack(TM):  All the god descriptions begin with "He/She is...".
local title = god_d.desc:find('^She') and 'Milady' or 'Milord'

local q = game.player:hasQuest('relic')
local n_quests_done = q and q:num_done() or 0

local quips = {
  {
    'Gah!  Jeez, not so loud...',
    'What, again?',
    "You can't be serious...",
    '...Wait, how does a god "lose" a temple?',
    'Oh, good grief, how many of these "lost temples" are there?',
  },
  {
    'Yyyeah, kinda busy right now, can I maybe do this later?',
    "Yeah, yeah, I remember this part.  I'm in.",
    "#{italic}#*sigh*#{normal}# Okay, you're serious.  So, where's the temple this time?",
    [[Seriously, it's a great big building.  You don't just "misplace" something like that.]],
    'Oh.  Well, finally.',
  },
  {
    'But... oh, fine.',
    'Right, right...',
    '#{italic}#*scribbles notes*#{normal}#  Got it.',
    "...You're not even listening to me, are you?",
    "And I'll be glad to see the back end of this whole business...",
  }
}

local stage2_txt = {
  [[One of my lost temples has recently been rediscovered.  Hidden within is a piece of one of my lost relics.  I want you to go and retrieve this piece.]],
  [[Another of my lost temples has recently been rediscovered.  Another piece of my lost relic is surely hidden within it.  You are to go and retrieve it.]],
  [[The last of my lost temples has been rediscovered.  With the last piece from this temple, my lost relic will finally be complete.  Go and retrieve this piece for me.]],
}

local function grant_quest(npc, player, dlg)
  local q = player:hasQuest('relic')
  if not q then
    player:grantQuest('relic')
    q = player:hasQuest('relic')
  else
    local n = q:num_done()
    player:setQuestStatus('relic', engine.Quest.PENDING, n+1)
  end
  if q then
    -- Sneaky Hack(TM):  Manually munge the next chat object's text to
    -- include a description of the quest temple location.
    local chat = dlg.chat:get('stage3')
    chat.text = [[Excellent!  You will find the temple ]]..q:temple_loc_desc(player)..[[.  Go now and quest in my name!]]
  end
  return 'stage3'
end

newChat {
  id = 'welcome',
  text = [[#LIGHT_GREEN#Suddenly the voice of @playerdescriptor.god@ booms out:#LAST#
@playername@!  I have a task for you.]],
  answers = {
    { 'Yes, '..title..'?', jump='stage2' },
    { quips[1][n_quests_done+1], jump='stage2' },
  },
}

newChat {
  id = 'stage2',
  text = stage2_txt[n_quests_done == 0 and 1 or n_quests_done == 4 and 3 or 2],
  answers = {
    { 'Of course, '..title..'.', action=grant_quest, jump='stage3' },
    { quips[2][n_quests_done+1], action=grant_quest, jump='stage3' },
  },
}

newChat {
  id = 'stage3',
  text = [[#RED#ERROR:  If you are seeing this text, something has gone wrong.#LAST#]],
  answers = {
    { 'As you bid, '..title..'.' },
    { quips[3][n_quests_done+1] },
  },
}

return 'welcome'
