-- $Id: melee-styles.lua 2379 2016-05-01 23:30:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

newMeleeStyle {
  id = 'WEAPON_COMBAT',
  desc = 'Weapon combat',
  skill = 'WEAPONMASTERY',
  one_blow = PlayerCombat.oneBlowWithWeapon,
  num_blows = PlayerCombat.numBlowsWithWeapon,
  all_weapons = function(src)
    local wpns = src:getInven('MAIN_HAND')
    -- Dummy weapon if we're not wielding anything.
    if #wpns == 0 then
      return {
	{ combat={ 1, 1 }, hasFlag=function(_, _, _) return false end },
      }
    end
    -- Clone this, because Nazgul effects can shatter weapons in the middle
    -- of our loop.
    return table.clone(wpns)
  end,
  on_deselect = function(def, src)
    local inven = src:getInven('MAIN_HAND')
    for _, o in ipairs(inven) do
      if o:check('on_prermoveobject', src, inven) or o:check('on_cantakeoff', src, inven) then
	return 'you are wielding one or more cursed weapons'
      end
    end
    -- Now that we know we can safely unwield all our weapons, do so.
    while #inven > 0 do src:doTakeoff('MAIN_HAND', 1, inven[1]) end
    return nil
  end,
  dmg_desc = PlayerCombat.dmgDescWeapon,
}

newMeleeStyle {
  id = 'BAREHANDED_COMBAT',
  desc = 'Barehanded combat',
  skill = 'BAREHAND_COMBAT',
  one_blow = PlayerCombat.oneBlowBarehand,
  num_blows = function(src, _)
    local wpns = src:getInven('MAIN_HAND')
    if wpns and #wpns > 0 then return 1 end
    local sk = src:getSkillLevel('BAREHAND_COMBAT')
    -- This gives us 1=1/10=2/20=3/30=4/35=5/40=6/45=7/50=8
    local bonus = 1 + math.floor(sk/10) + math.max(0, math.floor((sk-25)/10))
    if src:armorWeight() > 10 + sk*4/10 then
      bonus = math.floor(bonus/2)
    end
    return 1 + bonus
  end,
  all_weapons = function(src)
    local sk = src:getSkillLevel('BAREHAND_COMBAT')
    local wpns = src:getInven('MAIN_HAND')
    local th, td = 0, 0
    if (not wpns or #wpns == 0) and src:armorWeight() > 10 + sk*4/10 then
      th = math.floor(sk/3)
      td = th
    end
    return { { combat = { 1, 1 }, to_h = th, to_d = td } }
  end,
  dmg_desc = PlayerCombat.dmgDescBarehand,
  can_use = function(def, src)
    return src:getSkillLevel('BAREHAND_COMBAT') > 0
  end,
}

newMeleeStyle {
  id = 'BEARFORM_COMBAT',
  desc = 'Bearform combat',
  skill = 'BEARFORM_COMBAT',
  one_blow = PlayerCombat.oneBlowBearform,
  num_blows = function(src, _)
    return 2 + src:getSkillLevel('BEARFORM_COMBAT', 5)
  end,
  all_weapons = function(src)
    local sk = src:getSkillLevel('BEARFORM_COMBAT')
    return { { combat = { 1, 1 }, to_h = -math.floor(sk/5), to_d = math.floor(sk/2) } }
  end,
  dmg_desc = PlayerCombat.dmgDescBearform,
  can_use = function(def, src)
    return src:getSkillLevel('BEARFORM_COMBAT') > 0 and
	   src.inBearForm and src:inBearForm()
  end,
}
