-- $Id: bubble_vault.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_bubble_vault() in src/generate.c, to build a bubble
-- vault as a subtype of the 'random_vault' room type (q.v.).
newRoom {
  id = 'bubble_vault',
  no_random_gen = true,
  -- Number of "bubbles"
  n_bubbles = 10,
  build_special = function(self, gen, ctr, xsize, ysize)
    local xhsize, yhsize = math.floor(xsize/2), math.floor(ysize/2)

    -- Bubble centers.
    local ctrs = { { x=1+rng.range(1, xsize-3), y=1+rng.range(ysize-3) } }
    for i = 2, self.n_bubbles do
      -- Try 2000 times to get a bubble center that we don't already have.
      local x, y
      for _ = 1, 2000 do
	x, y = 1+rng.range(1, xsize-3), 1+rng.range(ysize-3)
	local any_hit = false
	for _, cc in ipairs(ctrs) do
	  any_hit = any_hit or (cc.x == x and cc.y == y)
	end
	if not any_hit then break end
	x, y = nil, nil
      end
      -- No luck?  Punt.
      if not x or not y then return false end
      ctrs[i] = { x=x, y=y }
    end

    -- Outer wall.
    gen:build_rectangle(ctr.x - xhsize, ctr.y - yhsize, ctr.x - xhsize + xsize - 1, ctr.y - yhsize + ysize - 1, 'outer_wall', false, { 'room', 'vault' })
    -- Fill in middle with bubbles.
    for x = 1, xsize - 1 do
      for y = 1, ysize - 1 do
	-- Get distance to two closest bubble centers.
	local min1 = core.fov.distance(x, y, ctrs[1].x, ctrs[1].y)
	local min2 = core.fov.distance(x, y, ctrs[2].x, ctrs[2].y)
	min1, min2 = math.min(min1, min2), math.max(min1, min2)
	for i = 3, #ctrs do
	  local d = core.fov.distance(x, y, ctrs[i].x, ctrs[i].y)
	  if d < min1 then
	    min1, min2 = d, min1
	  elseif d < min2 then
	    min2 = d
	  end
	end
	local tag
	if min2 - min1 <= 2 and min1 >= 3 then
	  -- The T2 code describes this as "Boundary at midpoint+ not at
	  -- inner region of bubble"
	  tag = 'fill'
	else
	  -- Otherwise, we're in a bubble.
	  tag = floor
	end
	local xx, yy = x + ctr.x - xhsize, y + ctr.y - yhsize
	gen.map(xx, yy, Map.TERRAIN, gen:resolve(tag, nil, false, gen.dun_lev))
	gen.map.attrs(xx, yy, 'room', true)
	gen.map.attrs(xx, yy, 'vault', true)
      end
    end

    -- Try to add some random doors
    for _ = 1, 500 do
      local x = rng.range(1, xsize-3) - xhsize + ctr.x + 1
      local y = rng.range(1, ysize-3) - yhsize + ctr.y + 1
      gen:try_vault_door(x, y)
    end

    -- And fill with monsters and treasure, at low power.
    gen:fill_treasure(ctr.x - xhsize + 1, ctr.y - yhsize + 1, ctr.x - xhsize + xsize - 2, ctr.y - yhsize + ysize - 2, rng.range(1, 5))
    return true
  end,
}
