-- $Id: cross.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_type3() in src/generate.c, to build a cross-shaped
-- room.
newRoom {
  id = 'cross',
  min_depth = 3,	-- cf. roomdep[] in src/generate.c
  wgt_unusual = 20,	-- cf. src/generate.c lines 6956-6957
  build = function(self, gen, gx, gy)
    local ctr = gen:roomAlloc(25, 11, false, gx, gy)
    if not ctr then return false end

    -- This results in a 3x3 cross.
    local wx, wy = 1, 1
    -- Vertical and horizontal size.
    local dy, dx = rng.range(3, 4), rng.range(3, 11)
    -- Extents for the "vertical" part of the cross...
    local x1a, y1a, x2a, y2a = ctr.x - wx, ctr.y - dy, ctr.x + wx, ctr.y + dy
    -- ...and the "horizontal" part
    local x1b, y1b, x2b, y2b = ctr.x - dx, ctr.y - wy, ctr.x + dx, ctr.y + wy

    local lit = gen.dun_lev <= rng.range(1, 25)

    -- First lay the walls for the two rectangular parts of the "cross".
    gen:build_rectangle(x1a-1, y1a-1, x2a+1, y2a+1, 'outer_wall', lit, {'room'})
    gen:build_rectangle(x1b-1, y1b-1, x2b+1, y2b+1, 'outer_wall', lit, {'room'})

    -- Then overlay floors for both parts, clobbering the walls as needed.
    gen:build_floor(x1a, y1a, x2a, y2a, 'floor', lit, {'room'})
    gen:build_floor(x1b, y1b, x2b, y2b, 'floor', lit, {'room'})

    -- Now the center bit, if any.
    local k = rng.range(1, 4)
    if k == 1 then
      -- Solid middle pillar.
      for y = y1b, y2b do
	for x = x1a, x2a do
	  gen.map(x, y, Map.TERRAIN, gen:resolve('inner_wall', false, gen.cur_lev))
	end
      end
    elseif k == 2 then
      -- Inner treasure vault.
      gen:build_rectangle(x1a, y2b, x2a, y2b, 'inner_wall', lit, {'room'})
      -- Secret door on random wall.
      gen:place_rect_secret_door(x1a, y2b, x2a, y2b, ctr.x, ctr.y)
      -- Treasure.
      gen:place_object(ctr.x, ctr.y)
      -- Guardian monster.
      -- [We follow the T2 code here, which inexplicably tries to place
      --  multiple monsters in an area with only one open spot...]
      gen:place_vault_mon(ctr.x, ctr.y, rng.range(3, 4))
      -- Traps.
      gen:place_vault_traps(ctr.x, ctr.y, 4, 4, rng.range(2, 4))
    elseif k == 3 then
      -- More elaborate stuff.
      if rng.chance(3) then
	-- "Box in" the center, punctured with either secret doors or open
	-- floor.
	gen:build_rectangle(x1a-1, y2b-1, x2a+1, y2b+1, 'inner_wall', lit, {'room'})
	if rng.chance(3) then
	  gen:place_secret_door(ctr.x, y1b-1)
	  gen:place_secret_door(ctr.x, y2b+1)
	  gen:place_secret_door(x1a-1, ctr.y)
	  gen:place_secret_door(x2a+1, ctr.y)
	else
	  gen.map(ctr.x, y1b-1, Map.TERRAIN, gen:resolve('floor', false, gen.cur_lev))
	  gen.map(ctr.x, y2b+1, Map.TERRAIN, gen:resolve('floor', false, gen.cur_lev))
	  gen.map(x1a-1, ctr.y, Map.TERRAIN, gen:resolve('floor', false, gen.cur_lev))
	  gen.map(x2a+1, ctr.y, Map.TERRAIN, gen:resolve('floor', false, gen.cur_lev))
	end
      elseif rng.chance(3) then
	-- "Plus"-shaped pillar.
	gen.map(ctr.x, ctr.y, Map.TERRAIN, gen:resolve('inner_wall', false, gen.cur_lev))
	gen.map(ctr.x, y1b, Map.TERRAIN, gen:resolve('inner_wall', false, gen.cur_lev))
	gen.map(ctr.x, y1b, Map.TERRAIN, gen:resolve('inner_wall', false, gen.cur_lev))
	gen.map(x1a, ctr.y, Map.TERRAIN, gen:resolve('inner_wall', false, gen.cur_lev))
	gen.map(x2a, ctr.y, Map.TERRAIN, gen:resolve('inner_wall', false, gen.cur_lev))
      elseif rng.chance(3) then
	-- Single pillar.
	gen.map(ctr.x, ctr.y, Map.TERRAIN, gen:resolve('inner_wall', false, gen.cur_lev))
      end
    end
    return true
  end,
}
