-- $Id: fractal_cave.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_type10() in src/generate.c, to build a fractal-
-- generated cave.
newRoom {
  id = 'fractal_cave',
  min_depth = 3,	-- cf. roomdep[] in src/generate.c
  wgt_unusual = 15,	-- cf. src/generate.c lines 6962-6963
  build = function(self, gen, gx, gy)
    -- Random size, enforced even.
    local xsize, ysize = rng.range(1, 22)*2 + 6, rng.range(1, 15)*2 + 6

    local ctr = gen:roomAlloc(xsize+1, ysize+1, false, gx, gy)
    if not ctr then return false end

    local done, lit = false, gen.dun_lev <= rng.range(1, 25)
    while not done do
      -- *sigh* Lua has no equivalent of C's << operator.
      local grd = util.lshift(1, rng.range(1, 4))
      -- Want average of about 16
      local roug = rng.range(1, 8) * rng.range(1, 4)
      -- About size/2
      local cutoff = rng.range(1, math.floor(xsize/4)) + 
		     rng.range(1, math.floor(ysize/4)) + 
		     rng.range(1, math.floor(xsize/4)) + 
		     rng.range(1, math.floor(ysize/4))

      local hmap = gen.generate_hmap(ctr.x, ctr.y, xsize, ysize, grd, roug, cutoff)
      done = gen:generate_fracave(ctr.x, ctr.y, xsize, ysize, hmap, cutoff, lit, true)
    end
    return true
  end,
}
