-- $Id: large.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_type4() in src/generate.c, to build a monster-pit-
-- shaped room with some internal features.
newRoom {
  id = 'large',
  min_depth = 3,	-- cf. roomdep[] in src/generate.c
  wgt_unusual = 25,	-- cf. src/generate.c lines 6953-6954
  build = function(self, gen, gx, gy)
    local ctr = gen:roomAlloc(25, 11, false, gx, gy)
    if not ctr then return false end

    local x1, y1, x2, y2 = ctr.x - 11, ctr.y - 4, ctr.x + 11, ctr.y + 4

    local lit = gen.dun_lev <= rng.range(1, 25)

    -- Full floor under the room.
    gen:build_floor(x1-1, y1-1, x2+1, y2+1, 'floor', lit, {'room'})
    -- Outer walls.
    gen:build_rectangle(x1-1, y1-1, x2+1, y2+1, 'outer_wall', lit, {'room'})

    -- Inner walls.
    x1, y1, x2, y2 = x1+2, y1+2, x2-2, y2-2
    gen:build_rectangle(x1-1, y1-1, x2+1, y2+1, 'inner_wall', lit, {'room'})

    -- Now the interesting stuff.
    local k = rng.range(1, 5)
    -- All but the four-subroom variant have a single secret door in the
    -- middle of one of the inner walls.
    if k ~= 5 then
      gen:place_rect_secret_door(x1-1, y1-1, x2+1, y2+1, ctr.x, ctr.y)
    end
    if k == 1 then
      -- Single monster.
      gen:place_vault_mon(ctr.x, ctr.y, 1)
    elseif k == 2 then
      -- Mini-vault
      gen:build_rectangle(ctr.x-1, ctr.y-1, ctr.x+1, ctr.y+1, 'inner_wall', lit, {'room'})
      -- Locked door on one side of mini-vault.
      local j = rng.range(1, 4)
      local lx = j == 3 and ctr.x - 1 or j == 4 and ctr.x + 1 or ctr.x
      local ly = j == 1 and ctr.y - 1 or j == 2 and ctr.y + 1 or ctr.y
      -- We count on resolvers in LOCKED_DOOR to set door_lock power.
      gen.map(lx, ly, Map.TERRAIN, gen:resolve('LOCKED_DOOR', nil, true))
      -- Single monster in mini-vault
      -- [We follow the T2 code here, which inexplicably tries to place
      --  multiple monsters in an area with only one open spot...]
      gen:place_vault_mon(ctr.x, ctr.y, rng.range(3, 4))
      if rng.percent(80) then
	-- Single object in mini-vault
	gen:place_object(ctr.x, ctr.y)
      else
	-- TODO Up/down stair in mini-vault
	-- [cf. place_random_stairs() in src/generate.c]
      end
      -- Traps
      gen:place_vault_traps(ctr.x, ctr.y, 10, 4, rng.range(3, 5))
    elseif k == 3 then
      -- Large inner pillar.
      local function pillar(cx, cy)
	for y = cy - 1, cy + 1 do
	  for x = cx - 1, cx + 1 do
	    gen.map(x, y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, self.cur_lev))
	  end
	end
      end
      pillar(ctr.x, ctr.y)
      if rng.chance(2) then
	-- Two more large inner pillars
	local tmp = rng.range(4, 5)
	pillar(ctr.x - tmp, ctr.y)
	pillar(ctr.x + tmp, ctr.y)
      end
      if rng.chance(3) then
	-- Inner rooms.
	gen:build_rectangle(ctr.x-5, ctr.y-1, ctr.x+5, ctr.y+1, 'inner_wall', lit, {'room'})
	-- With secret doors, randomly on north/south walls.
	gen:place_secret_door(ctr.x - 3, ctr.y + (rng.chance(2) and 1 or -1))
	gen:place_secret_door(ctr.x + 3, ctr.y + (rng.chance(2) and 1 or -1))
	-- Monsters in rooms
	gen:place_vault_mon(ctr.x - 2, ctr.y, rng.range(1, 2))
	gen:place_vault_mon(ctr.x + 2, ctr.y, rng.range(1, 2))
	-- Objects in rooms
	if rng.chance(3) then gen:place_object(ctr.x - 2, ctr.y) end
	if rng.chance(3) then gen:place_object(ctr.x + 2, ctr.y) end
      end
    elseif k == 4 then
      -- "Checkerboard" pit.
      for y = y1, y2 do
	for x = x1, x2 do
	  if (x + y) % 2 == 1 then
	    gen.map(x, y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, self.cur_lev))
	  end
	end
      end
      -- Monsters
      gen:place_vault_mon(ctr.x - 5, ctr.y, rng.range(1, 3))
      gen:place_vault_mon(ctr.x + 5, ctr.y, rng.range(1, 3))
      -- Traps
      gen:place_vault_mon(ctr.x - 3, ctr.y, 8, 2, rng.range(1, 3))
      gen:place_vault_mon(ctr.x + 3, ctr.y, 8, 2, rng.range(1, 3))
      -- Objects
      gen:place_vault_obj(ctr.x, ctr.y, 3)
    else
      -- Four subrooms
      for y = y1, y2 do
	gen.map(ctr.x, y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, self.cur_lev))
      end
      for x = x1, x2 do
	gen.map(x, ctr.y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, self.cur_lev))
      end
      -- Secret doors, on north/south walls or east/west walls.
      if rng.chance(2) then
	local i = rng.range(1, 10)
	gen:place_secret_door(ctr.x - i, y1 - 1)
	gen:place_secret_door(ctr.x + i, y1 - 1)
	gen:place_secret_door(ctr.x - i, y2 + 1)
	gen:place_secret_door(ctr.x + i, y2 + 1)
      else
	local i = rng.range(1, 3)
	gen:place_secret_door(x1 - 1, ctr.y - i)
	gen:place_secret_door(x1 - 1, ctr.y + i)
	gen:place_secret_door(x2 + 1, ctr.y - i)
	gen:place_secret_door(x2 + 1, ctr.y + i)
      end
      -- Objects, centered at the center of the cross
      gen:place_vault_obj(ctr.x, ctr.y, rng.range(3, 4))
      -- Monsters
      gen:place_vault_mon(ctr.x - 4, ctr.y - 1, 8, 2, rng.range(1, 4))
      gen:place_vault_mon(ctr.x + 4, ctr.y - 1, 8, 2, rng.range(1, 4))
      gen:place_vault_mon(ctr.x - 4, ctr.y + 1, 8, 2, rng.range(1, 4))
      gen:place_vault_mon(ctr.x + 4, ctr.y + 1, 8, 2, rng.range(1, 4))
    end
    return true
  end,
}

