-- $Id: monster_nest.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_type5() in src/generate.c, to build a monster nest.
newRoom {
  id = 'monster_nest',
  min_depth = 5,		-- cf. roomdep[] in src/generate.c
  wgt_very_unusual = 15,	-- cf. src/generate.c lines 6942-6943
  crowded = true,
  filters = {
    jelly = {
      -- cf. vault_aux_jelly() in src/generate.c
      precog_name = 'jelly',
      add_levels = 10,
      special = function(e)
	return string.find('ijm,', e.display, 1, true) and not e:has('EVIL') and not e.unique
      end,
    },
    treasure = {
      -- cf. vault_aux_treasure() in src/generate.c
      precog_name = 'treasure',
      add_levels = 10,
      special = function(e)
	return string.find('!|$?=', e.display, 1, true) and not e.unique
      end,
    },
    kennel = {
      -- cf. vault_aux_kennel() in src/generate.c
      precog_name = 'kennel',
      add_levels = 10,
      special = function(e)
	return not e.unique and (e.display == 'Z' or e.display == 'C')
      end,
    },
    animal = {
      -- cf. vault_aux_animal() in src/generate.c
      precog_name = 'animal',
      add_levels = 10,
      special = function(e) return e:has('ANIMAL') and not e.unique end,
    },
    chapel = {
      -- cf. vault_aux_chapel() in src/generate.c
      precog_name = 'chapel',
      add_levels = 10,
      special = function(e)
	return not e.unique and (e.display == 'A' or e.name:find('riest', 1, true))
      end,
    },
    undead = {
      -- cf. vault_aux_undead() in src/generate.c
      precog_name = 'undead',
      add_levels = 10,
      special = function(e) return e:has('UNDEAD') and not e.unique end,
    },
  },
  build = function(self, gen, gx, gy)
    local ctr = gen:roomAlloc(25, 11, true, gx, gy)
    if not ctr then return false end

    -- Outer room extents
    local x1, x2, y1, y2 = ctr.x - 11, ctr.x + 11, ctr.y - 4, ctr.y + 4
    -- Floor
    gen:build_floor(x1, y1, x2, y2, 'floor', false, {'room'})
    -- Walls
    gen:build_rectangle(x1-1, y1-1, x2+1, y2+1, 'outer_wall', false, {'room'})

    -- Inner room extents.
    x1, y1, x2, y2 = x1+2, y1+2, x2-2, y2-2
    -- Walls
    gen:build_rectangle(x1-1, y1-1, x2+1, y2+1, 'inner_wall', false, {'room'})
    -- Secret door on one wall.
    gen:place_rect_secret_door(x1-1, y1-1, x2+1, y2+1, ctr.x, ctr.y)

    -- Choose a nest type.
    local tmp = rng.range(1, gen.dun_lev)
    local filter = nil
    if tmp < 25 and rng.chance(2) then
      -- Pick a monster to clone. (cf. src/generate.c lines 3008-3032)
      local tmpl
      repeat
	tmpl = rng.table(gen.zone.npc_list)
      until not tmpl.unique and not tmpl.special_gene and tmpl.level + rng.range(1, 5) <= gen.dun_lev + rng.range(1, 5)
      if gen.dun_lev >= 25 + rng.range(1, 15) and rng.chance(2) then
	filter = {
	  precog_name = 'symbol clone',
	  add_levels = 10,
	  special = function(e)
	    return not e.unique and e.display == tmpl.display
	  end,
	}
      else
	filter = {
	  precog_name = 'clone',
	  name = tmpl.name,
	}
      end
    else
      filter = tmp < 25 and self.filters.jelly or
	       tmp < 50 and self.filters.treasure or
	       tmp < 65 and self.filters[rng.chance(3) and 'kennel' or 'animal'] or
			    self.filters[rng.chance(3) and 'chapel' or 'undead']
    end

    -- Pick some monsters with our filter.
    local what, empty = {}, false
    for i = 1, 64 do
      what[i] = gen.zone:chooseEntity(gen.level, 'actor', filter, nil, true)
      empty = empty or not what[i]
    end

    -- Pursuant to the T2 code, we only fill the room if we were able to
    -- fill all 64 slots.
    if empty then return false end

    gen:report_precog('Monster nest (%s)', filter and filter.precog_name or '???')

    -- Increase level rating.
    gen.level.rating = (gen.level.rating or 0) + 10
    -- Also, depending on depth, possibly flag the level as "special".
    if gen.dun_lev <= 40 and rng.range(1, gen.dun_lev*gen.dun_lev + 50) < 300 then
      gen.level.special_feeling = true
    end

    -- Now actually place monsters in the room.
    local agen = gen.actor_gen
    for y = ctr.y - 2, ctr.y + 2 do
      for x = ctr.x - 9, ctr.x + 9 do
	local which = rng.table(what)
	agen:generateOne{ name = which.define_as, x=x, y=y, no_grp=true }
      end
    end
    return true
  end,
}
