-- $Id: overlapping.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_type2() in src/generate.c, to build a room of two
-- overlapping rectangles.
newRoom {
  id = 'overlapping',
  min_depth = 1,	-- cf. roomdep[] in src/generate.c
  wgt_unusual = 20,	-- cf. src/generate.c lines 6959-6960
  build = function(self, gen, gx, gy)
    local ctr = gen:roomAlloc(25, 11, false, gx, gy)
    if not ctr then return false end

    -- Extents of first rectangle.
    local y1a = ctr.y - rng.range(1, 4)
    local y2a = ctr.y + rng.range(1, 3)
    local x1a = ctr.x - rng.range(1, 14)
    local x2a = ctr.x + rng.range(1, 6)

    -- Extents of first rectangle.
    local y1b = ctr.y - rng.range(1, 3)
    local y2b = ctr.y + rng.range(1, 4)
    local x1b = ctr.x - rng.range(1, 6)
    local x2b = ctr.x + rng.range(1, 14)

    local lit = gen.dun_lev <= rng.range(1, 25)

    -- First lay the walls around rooms 'a' and 'b'.
    gen:build_rectangle(x1a-1, y1a-1, x2a+1, y2a+1, 'outer_wall', lit, {'room'})
    gen:build_rectangle(x1b-1, y1b-1, x2b+1, y2b+1, 'outer_wall', lit, {'room'})

    -- Then overlay the floors for both rooms, clobbering the walls as needed.
    gen:build_floor(x1a, y1a, x2a, y2a, 'floor', lit, {'room'})
    gen:build_floor(x1b, y1b, x2b, y2b, 'floor', lit, {'room'})
    return true
  end,
}
