-- $Id: random_vault.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_type11() in src/generate.c, to build a random
-- vault.
newRoom {
  id = 'random_vault',
  min_depth = 10,		-- cf. roomdep[] in src/generate.c
  wgt_very_unusual = 5,		-- cf. src/generate.c lines 6948-6949
  -- Random vault types.
  types = {
    'bubble_vault', 'room_vault', 'cave_vault', 'maze_vault',
    'mini_c_vault', 'castle_vault', 'target_vault' },
  build = function(self, gen, gx, gy)
    local xsize, ysize = rng.range(23, 44), rng.range(12, 22)
    local ctr = gen:roomAlloc(xsize+2, ysize+2, false, gx, gy)
    if not ctr then return false end

    -- Boost rating.
    gen.level.rating = (gen.level.rating or 0) + 10
    -- Also, depending on depth, possibly flag the level as "special".
    local dl40 = gen.dun_lev - 40
    if gen.dun_lev <= 50 and rng.range(1, dl40*dl40 + 1) < 400 then
      gen.level.special_feeling = true
    end

    -- Generate a random vault type.
    local subroom = gen.room_types[rng.table(self.types)]
    print(("[MAPGEN]     random_vault using subroom '%s'"):format(subroom.id))
    return subroom:build_special(gen, ctr, xsize, ysize)
  end,
}

-- Load the subroom definitions.
load('/data/rooms/bubble_vault.lua')
load('/data/rooms/room_vault.lua')
load('/data/rooms/cave_vault.lua')
load('/data/rooms/maze_vault.lua')
load('/data/rooms/mini_c_vault.lua')
load('/data/rooms/castle_vault.lua')
load('/data/rooms/target_vault.lua')
