-- $Id: room_vault.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_room_vault() in src/generate.c, to build a room
-- vault as a subtype of the 'random_vault' room type (q.v.).
newRoom {
  id = 'room_vault',
  no_random_gen = true,
  build_special = function(self, gen, ctr, xsize, ysize)
    local xhsize, yhsize = math.floor(xsize/2), math.floor(ysize/2)

    -- Fill area so we don't get problems with arena levels.
    for x = ctr.x - xhsize, ctr.x - xhsize + xsize - 1 do
      for y = ctr.y - yhsize, ctr.y - yhsize + ysize - 1 do
	gen.map(x, y, Map.TERRAIN, gen:resolve('outer_wall', nil, false, gen.dun_lev))
	gen.map:unattr(x, y, 'vault')
      end
    end

    -- cf build_room() in src/generate.c()
    local function build_room(x1, y1, x2, y2)
      -- Check if rectangle has no width.
      if x1 == x2 or y1 == y2 then return end
      -- Swap boundaries into correct order.
      x1, x2 = math.min(x1, x2), math.max(x1, x2)
      y1, y2 = math.min(y1, y2), math.max(y1, y2)

      gen:build_rectangle(x1, y1, x2, y2, 'outer_wall', false, {'room', 'vault'})
      -- Replace any 'outer_wall' grids inside the "room" rectangle with
      -- floor.
      for x = x1 + 1, x2 - 1 do
	for y = y1 + 1, y2 - 1 do
	  local g = gen.map(x, y, Map.TERRAIN)
	  if g.define_as == gen.data.outer_wall then
	    gen.map(x, y, Map.TERRAIN, gen:resolve(floor, nil, false, gen.dun_lev))
	  end
	  gen.map.attrs(x, y, 'room', true)
	  gen.map.attrs(x, y, 'vault', true)
	end
      end
    end

    -- Add ten random rooms.
    for _ = 1, 10 do
      local x1 = rng.range(1, xhsize)*2 + ctr.x - xhsize
      local y1 = rng.range(1, yhsize)*2 + ctr.y - yhsize
      local x2 = rng.range(1, xhsize)*2 + ctr.x - xhsize
      local y2 = rng.range(1, yhsize)*2 + ctr.y - yhsize
      build_room(x1, y1, x2, y2)
    end

    -- Convert walls to normal dungeon wall. [cf. convert_extra() in
    -- src/generate.c]
    for x = ctr.x - xhsize, ctr.x - xhsize + xsize - 1 do
      for y = ctr.y - yhsize, ctr.y - yhsize + ysize - 1 do
	local g = gen.map(x, y, Map.TERRAIN)
	if g.define_as == gen.data.outer_wall then
	  gen.map(x, y, Map.TERRAIN, gen:resolve('fill', nil, false, gen.dun_lev))
	end
      end
    end

    -- Add some random doors.
    for _ = 1, 500 do
      local x1 = rng.range(1, xsize - 2) - xhsize + ctr.x + 1
      local y1 = rng.range(1, ysize - 2) - yhsize + ctr.y + 1
      gen:try_vault_door(x1, y1)
    end

    -- Fill with monsters and treasure, high difficulty.
    gen:fill_treasure(ctr.x - xhsize + 1, ctr.y - yhsize + 1, ctr.x - xhsize + xsize + 1, ctr.y - yhsize + ysize - 1, rng.range(6, 10))
    return true
  end,
}
