-- $Id: target_vault.lua 2452 2016-08-07 23:30:36Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Roughly mimics build_target_vault() in src/generate.c, to build a target
-- vault as a subtype of the 'random_vault' room type (q.v.).
newRoom {
  id = 'target_vault',
  no_random_gen = true,
  build_special = function(self, gen, ctr, xsize, ysize)
    -- Build a random metric
    local h1, h2 = rng.range(-15, 16), rng.range(1, 16)
    local h3, h4 = rng.range(1, 32), rng.range(-15, 16)
    -- Work out outer radius
    local rad = math.floor(math.min(xsize, ysize)/2)

    -- Make floor
    for x = ctr.x - rad, ctr.x + rad do
      for y = ctr.y - rad, ctr.y + rad do
	-- Clear room flag, for use with :add_outer_wall()
	gen.map:unattr(x, y, 'room')
	gen.map.attrs(x, y, 'vault', true)
	-- Floor or wall, depending on distance from center.
	if gen:dist2(ctr.x, ctr.y, x, y, h1, h2, h3, h4) <= rad - 1 then
	  gen.map(x, y, Map.TERRAIN, gen:resolve('floor', nil, false, gen.cur_lev))
	else
	  -- :add_outer_wall() lools for WALL_XTRA's .xtra_hack.
	  gen.map(x, y, Map.TERRAIN, gen.zone.grid_list.WALL_XTRA)
	end
      end
    end
    -- Outer walls.
    gen:build_rectangle(ctr.x - rad, ctr.y - rad, ctr.x + rad, ctr.y + rad, 'outer_wall')

    -- Find visible outer walls and set them to outer_wall.
    gen:add_outer_wall(ctr.x, ctr.y, false, ctr.x - rad - 1, ctr.y - rad - 1, ctr.x + rad + 1, ctr.y + rad + 1)

    -- Add inner wall.
    local r2 = math.floor(rad/2)
    for x = ctr.x - r2, ctr.x + r2 do
      for y = ctr.y - r2, ctr.y + r2 do
	if gen:dist2(ctr.x, ctr.y, x, y, h1, h2, h3, h4) == r2 then
	  gen.map(x, y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, gen.cur_lev))
	end
      end
    end

    -- Add perpendicular walls.
    for x = ctr.x - rad, ctr.x + rad do
      gen.map(x, ctr.y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, gen.cur_lev))
    end
    for y = ctr.y - rad, ctr.y + rad do
      gen.map(ctr.x, y, Map.TERRAIN, gen:resolve('inner_wall', nil, false, gen.cur_lev))
    end

    -- Make inner vault.
    gen:build_rectangle(ctr.x - 1, ctr.y - 1, ctr.x + 1, ctr.y + 1, 'inner_wall')
    gen.map(ctr.x, ctr.y, Map.TERRAIN, gen:resolve('floor', nil, false, gen.cur_lev))

    -- Add doors to vault.
    -- Get two distances so we can place doors relative to center.
    local d1 = math.floor((rad - 2)/4) + 1
    local d2 = r2 + d1

    get:try_vault_door(ctr.x + d1, ctr.y)
    get:try_vault_door(ctr.x + d2, ctr.y)
    get:try_vault_door(ctr.x - d1, ctr.y)
    get:try_vault_door(ctr.x - d2, ctr.y)
    get:try_vault_door(ctr.x, ctr.y + d1)
    get:try_vault_door(ctr.x, ctr.y + d2)
    get:try_vault_door(ctr.x, ctr.y - d1)
    get:try_vault_door(ctr.x, ctr.y - d2)

    -- Fill with stuff, medium dificulty.
    gen:fill_treasure(ctr.x - rad, ctr.y - rad, ctr.x + rad, ctr.y + rad, rng.range(4, 6))
    return true
  end,
}
