-- $Id: spells.lua 2669 2016-12-01 00:57:32Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

local Entity = require 'engine.Entity'

newTalentType {
  name = 'NPC spells',
  type = 'npc/spell',
  description = 'Spells that can be cast by monsters',
}

newTalent {
  name = 'Aggravate Monsters',
  short_name = 'NPC_SHRIEK',
  display_entity = Entity.new { image = 'talents/aggravate_monsters.png' },
  type = 'npc/spell',
  mode = 'activated',
  points = 0,
  player = { mana=1 },
  innate = true,	-- cf. spell.fail_prob()
  message = '@Source@ makes a high-pitched shriek.',
  action = function(self, t)
    spell.aggravate_near(self)
    return true
  end,
  info = 'Aggravates monsters near the caster.',
}

newTalent {
  name = 'Fire a Rocket',
  short_name = 'NPC_ROCKET',
  display_entity = Entity.new { image = 'talents/fire_a_rocket.png' },
  type = 'npc/spell',
  mode = 'activated',
  points = 0,
  player = { mana=40, major=true },
  range = 18,
  requires_target = true,
  innate = true,	-- cf. spell.fail_prob()
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Fires a rocket, doing %d blast damage in a radius of 2 around the target.]]
    return tmp:format(t.ball_damage(self))
  end,
  ball_damage = function(e) return math.min(e.life/4, 800) end,
  message = "@Source@ fires a rocket.",
  params = { r = 2, type = DamageType.ROCKET }
}

newTalent {
  name = 'Light Arrow',
  short_name = 'NPC_ARROW_1',
  display_entity = Entity.new { image = 'talents/light_arrow.png' },
  type = 'npc/spell',
  mode = 'activated',
  points = 0,
  player = { mana=1 },
  range = 18,
  requires_target = true,
  innate = true,	-- cf. spell.fail_prob()
  action = spell.npc.bolt_spell,
  info = 'Fires an arrow, doing 1d6 damage.',
  bolt_damage = function(e) return rng.dice(1, 6) end,
  message = "@Source@ fires an arrow.",
  params = { type = DamageType.PHYSICAL },
}

newTalent {
  name = 'Minor Arrow',
  short_name = 'NPC_ARROW_2',
  display_entity = Entity.new { image = 'talents/minor_arrow.png' },
  type = 'npc/spell',
  mode = 'activated',
  points = 0,
  player = { mana=3 },
  range = 18,
  requires_target = true,
  innate = true,	-- cf. spell.fail_prob()
  action = spell.npc.bolt_spell,
  info = 'Fires an arrow, doing 3d6 damage.',
  bolt_damage = function(e) return rng.dice(3, 6) end,
  message = "@Source@ fires an arrow!",
  params = { type = DamageType.PHYSICAL },
}

newTalent {
  name = 'Major Arrow',
  short_name = 'NPC_ARROW_3',
  display_entity = Entity.new { image = 'talents/major_arrow.png' },
  type = 'npc/spell',
  mode = 'activated',
  points = 0,
  player = { mana=7, major=true },
  range = 18,
  requires_target = true,
  innate = true,	-- cf. spell.fail_prob()
  action = spell.npc.bolt_spell,
  info = 'Fires an arrow, doing 5d6 damage.',
  bolt_damage = function(e) return rng.dice(5, 6) end,
  message = "@Source@ fires a missile.",
  params = { type = DamageType.PHYSICAL },
}

newTalent {
  name = 'Great Arrow',
  short_name = 'NPC_ARROW_4',
  display_entity = Entity.new { image = 'talents/great_arrow.png' },
  type = 'npc/spell',
  mode = 'activated',
  points = 0,
  player = { mana=9, major=true },
  range = 18,
  requires_target = true,
  innate = true,	-- cf. spell.fail_prob()
  action = spell.npc.bolt_spell,
  info = 'Fires an arrow, doing 7d6 damage.',
  bolt_damage = function(e) return rng.dice(7, 6) end,
  message = "@Source@ fires a missile!.",
  params = { type = DamageType.PHYSICAL },
}

newTalent {
  name = 'Nuke Ball',
  short_name = 'NPC_BA_NUKE',
  display_entity = Entity.new { image = 'talents/nuke_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  -- Looks like a spell, but it's in the RF4_ flag group, so it isn't.
  -- [cf. make_attack_spell() in src/melee2.c]
  --is_spell = true,
  points = 0,
  player = { mana=30, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  innate = true,	-- cf. spell.fail_prob()
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of radiation at the target, doing 10d6+%d damage in radius 2.]]
    return tmpl:format(e.level)
  end,
  ball_damage = function(e) return e.level + rng.dice(10, 6) end,
  message = "@Source@ casts a ball of radiation.",
  params = { r = 2, type = DamageType.NUKE }
}

newTalent {
  name = 'Chaos Ball',
  short_name = 'NPC_BA_CHAO',
  display_entity = Entity.new { image = 'talents/chaos_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  -- Looks like a spell, but it's in the RF4_ flag group, so it isn't.
  -- [cf. make_attack_spell() in src/melee2.c]
  --is_spell = true,
  points = 0,
  player = { mana=30, major=true },
  range = 18,
  direct_hit = true,
  innate = true,	-- cf. spell.fail_prob()
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of chaos at the target, doing 10d10+%d damage in radius 4.]]
    return tmpl:format(e.level*2)
  end,
  ball_damage = function(e) return e.level*2 + rng.dice(10, 10) end,
  message = "@Source@ invokes raw chaos.",
  params = { r = 4, type = DamageType.CHAOS }
}

newTalent {
  name = 'Acid Ball',
  short_name = 'NPC_BA_ACID',
  display_entity = Entity.new { image = 'talents/acid_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=8 },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of acid at the target, doing %d damage in radius 2.]]
    return tmpl:format(t.ball_damage(self))
  end,
  ball_damage = function(e) return e.level*3 + 15 end,
  message = "@Source@ casts an acid ball.",
  params = { r = 2, type = DamageType.ACID }
}

newTalent {
  name = 'Lightning Ball',
  short_name = 'NPC_BA_ELEC',
  display_entity = Entity.new { image = 'talents/lightning_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=8 },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of lightning at the target, doing 8+1d%d damage in radius 2.]]
    return tmpl:format(self.level*3/2)
  end,
  ball_damage = function(e) return rng.range(1, e.level*3/2) + 8 end,
  message = "@Source@ casts a lightning ball.",
  params = { r = 2, type = DamageType.ELEC }
}

newTalent {
  name = 'Fire Ball',
  short_name = 'NPC_BA_FIRE',
  display_entity = Entity.new { image = 'talents/fire_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=8 },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of fire at the target, doing 10+1d%d damage in radius 2.]]
    return tmpl:format(self.level*7/2)
  end,
  ball_damage = function(e) return rng.range(1, e.level*7/2) + 10 end,
  message = "@Source@ casts a fire ball.",
  params = { r = 2, type = DamageType.FIRE }
}

newTalent {
  name = 'Cold Ball',
  short_name = 'NPC_BA_COLD',
  display_entity = Entity.new { image = 'talents/cold_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=8 },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of frost at the target, doing 10+1d%d damage in radius 2.]]
    return tmpl:format(self.level*3/2)
  end,
  ball_damage = function(e) return rng.range(1, e.level*3/2) + 10 end,
  message = "@Source@ casts a frost ball.",
  params = { r = 2, type = DamageType.COLD }
}

newTalent {
  name = 'Poison Ball',
  short_name = 'NPC_BA_POIS',
  display_entity = Entity.new { image = 'talents/poison_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = [[Casts a ball of poison gas at the target, doing 12d2 damage in radius 2.]],
  ball_damage = function(e) return rng.dice(12, 2) end,
  message = "@Source@ casts a stinking cloud.",
  params = { r = 2, type = DamageType.POISON }
}

newTalent {
  name = 'Nether Ball',
  short_name = 'NPC_BA_NETH',
  display_entity = Entity.new { image = 'talents/nether_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of nether at the target, doing 10d10+%d damage in radius 2.]]
    return tmpl:format(self.level + 50)
  end,
  ball_damage = function(e) return e.level + rng.dice(10, 10) + 50 end,
  message = "@Source@ casts a nether ball.",
  params = { r = 2, type = DamageType.NETHER }
}

newTalent {
  name = 'Water Ball',
  short_name = 'NPC_BA_WATE',
  display_entity = Entity.new { image = 'talents/water_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a ball of water at the target, doing 50+1d%d damage in radius 4.]]
    return tmpl:format(self.level*5/2)
  end,
  ball_damage = function(e) return rng.range(1, e.level*5/2) + 50 end,
  message = "@Source@ gestures fluidly.",
  params = {
    r = 4, type = DamageType.WATER,
    target_msg = '#Target# is engulfed in a whirlpool!',
  }
}

newTalent {
  name = 'Mana Ball',
  short_name = 'NPC_BA_MANA',
  display_entity = Entity.new { image = 'talents/mana_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=50, major=true },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a mana storm around the target, doing 10d10+%d damage in radius 4.]]
    return tmpl:format(self.level*5)
  end,
  ball_damage = function(e) return e.level*5 + rng.dice(10, 10) end,
  message = "@Source@ invokes a mana storm.",
  params = { r = 4, type = DamageType.MANA }
}

newTalent {
  name = 'Darkness Ball',
  short_name = 'NPC_BA_DARK',
  display_entity = Entity.new { image = 'talents/darkness_ball.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  range = 18,
  direct_hit = true,
  requires_target = true,
  action = spell.npc.ball_spell,
  info = function(self, t)
    local tmpl = [[Casts a darkness storm around the target, doing 10d10+%d damage in radius 4.]]
    return tmpl:format(self.level*5)
  end,
  ball_damage = function(e) return e.level*5 + rng.dice(10, 10) end,
  message = "@Source@ invokes a darkness storm.",
  params = { r = 4, type = DamageType.MANA }
}

newTalent {
  name = 'Cause Light Wounds',
  short_name = 'NPC_CAUSE_1',
  display_entity = Entity.new { image = 'talents/cause_light_wounds.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.cause_wounds,
  info = 'Harms the target for 3d8 damage.',
  spell_damage = function(e) return rng.dice(3, 8) end,
  curse_chance = { 33, 0 },
  message = "@Source@ points at @target@ and curses.",
}

newTalent {
  name = 'Cause Medium Wounds',
  short_name = 'NPC_CAUSE_2',
  display_entity = Entity.new { image = 'talents/cause_medium_wounds.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=30 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.cause_wounds,
  info = 'Harms the target for 8d8 damage.',
  spell_damage = function(e) return rng.dice(8, 8) end,
  curse_chance = { 50, 5 },
  message = "@Source@ points at @target@ and curses horribly.",
}

newTalent {
  name = 'Cause Critical Wounds',
  short_name = 'NPC_CAUSE_3',
  display_entity = Entity.new { image = 'talents/cause_critical_wounds.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=35, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.cause_wounds,
  info = 'Harms the target for 10d15 damage.',
  spell_damage = function(e) return rng.dice(10, 15) end,
  curse_chance = { 80, 15 },
  message = "@Source@ points at @target@, incanting terribly!",
}

newTalent {
  name = 'Cause Mortal Wounds',
  short_name = 'NPC_CAUSE_4',
  display_entity = Entity.new { image = 'talents/cause_mortal_wounds.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=45, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.cause_wounds,
  info = 'Harms the target for 15d15 damage and cuts for 10d10 turns.',
  spell_damage = function(e) return rng.dice(15, 15) end,
  cut_duration = function() return rng.dice(10, 10) end,
  message = "@Source@ points at @target@, sreaming the word DIE!",
}

newTalent {
  name = 'Acid Bolt',
  short_name = 'NPC_BO_ACID',
  display_entity = Entity.new { image = 'talents/acid_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=5 },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of acid at the target, doing 7d8+%d damage.]]
    return tmpl:format(self.level/3)
  end,
  bolt_damage = function(e) return rng.dice(7, 8) + e.level/3 end,
  message = "@Source@ casts an acid bolt.",
  params = { type = DamageType.ACID },
}

newTalent {
  name = 'Lightning Bolt',
  short_name = 'NPC_BO_ELEC',
  display_entity = Entity.new { image = 'talents/lightning_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=5 },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of lightning at the target, doing 4d6+%d damage.]]
    return tmpl:format(self.level/3)
  end,
  bolt_damage = function(e) return rng.dice(4, 6) + e.level/3 end,
  message = "@Source@ casts a lightning bolt.",
  params = { type = DamageType.ELEC },
}

newTalent {
  name = 'Fire Bolt',
  short_name = 'NPC_BO_FIRE',
  display_entity = Entity.new { image = 'talents/fire_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=5 },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of fire at the target, doing 9d8+%d damage.]]
    return tmpl:format(self.level/3)
  end,
  bolt_damage = function(e) return rng.dice(9, 8) + e.level/3 end,
  message = "@Source@ casts a fire bolt.",
  params = { type = DamageType.FIRE },
}

newTalent {
  name = 'Cold Bolt',
  short_name = 'NPC_BO_COLD',
  display_entity = Entity.new { image = 'talents/cold_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=5 },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of frost at the target, doing 6d8+%d damage.]]
    return tmpl:format(self.level/3)
  end,
  bolt_damage = function(e) return rng.dice(6, 8) + e.level/3 end,
  message = "@Source@ casts a frost bolt.",
  params = { type = DamageType.COLD },
}

--[[ Not implemented, apparently.
newTalent {
  name = 'Poison Bolt',
  short_name = 'NPC_BO_POIS',
  display_entity = Entity.new { image = 'talents/npc_breathe.png' },
  type = 'npc/spell',
  mode = 'activated',
  points = 0,
  player = { mana=10, major=true },
  action = function(self, t)
  end,
}
]]

newTalent {
  name = 'Nether Bolt',
  short_name = 'NPC_BO_NETH',
  display_entity = Entity.new { image = 'talents/nether_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=15, major=true },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of nether at the target, doing 5d5+%d damage.]]
    return tmpl:format(self.level + 30)
  end,
  bolt_damage = function(e) return rng.dice(5, 5) + 30 + e.level end,
  message = "@Source@ casts a nether bolt.",
  params = { type = DamageType.NETHER },
}

newTalent {
  name = 'Water Bolt',
  short_name = 'NPC_BO_WATE',
  display_entity = Entity.new { image = 'talents/water_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of water at the target, doing 10d10+%d damage.]]
    return tmpl:format(self.level)
  end,
  bolt_damage = function(e) return rng.dice(10, 10) + e.level end,
  message = "@Source@ casts a water bolt.",
  params = { type = DamageType.WATER },
}

newTalent {
  name = 'Mana Bolt',
  short_name = 'NPC_BO_MANA',
  display_entity = Entity.new { image = 'talents/mana_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=25, major=true },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of water at the target, doing 1d%d+50 damage.]]
    return tmpl:format(self.level*7/2)
  end,
  bolt_damage = function(e) return rng.range(1, e.level*7/2) + 50 end,
  message = "@Source@ casts a mana bolt.",
  params = { type = DamageType.MANA },
}

newTalent {
  name = 'Plasma Bolt',
  short_name = 'NPC_BO_PLAS',
  display_entity = Entity.new { image = 'talents/plasma_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of plasma at the target, doing 8d7+%d damage.]]
    return tmpl:format(self.level + 10)
  end,
  bolt_damage = function(e) return rng.dice(8, 7) + 10 + e.level end,
  message = "@Source@ casts a plasma bolt.",
  params = { type = DamageType.PLASMA },
}

newTalent {
  name = 'Ice Bolt',
  short_name = 'NPC_BO_ICEE',
  display_entity = Entity.new { image = 'talents/ice_bolt.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a bolt of ice at the target, doing 6d6+%d damage.]]
    return tmpl:format(self.level)
  end,
  bolt_damage = function(e) return rng.dice(6, 6) + e.level end,
  message = "@Source@ casts an ice bolt.",
  params = { type = DamageType.ICE },
}

newTalent {
  name = 'Magic Missile',
  short_name = 'NPC_MISSILE',
  display_entity = Entity.new { image = 'talents/magic_missile.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=1 },
  range = 18,
  requires_target = true,
  action = spell.npc.bolt_spell,
  info = function(self, t)
    local tmpl = [[Casts a magic missile at the target, doing 2d6+%d damage.]]
    return tmpl:format(self.level/3)
  end,
  bolt_damage = function(e) return rng.dice(2, 6) + e.level/3 end,
  message = "@Source@ casts a magic missile.",
  params = { type = DamageType.MISSILE },
}

newTalent {
  name = 'Scare',
  short_name = 'NPC_SCARE',
  display_entity = Entity.new { image = 'talents/scare.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=4 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.effect_spell,
  info = 'Frightens the target for 3+1d4 turns.',
  message = '@Source@ casts a fearful illusion at @target@.',
  params = {
    eff_id = 'EFF_AFRAID', resist_flag = 'NO_FEAR',
    duration = function() return 3 + rng.range(1, 4) end,
    resist_msg = '#Target# refuses to be frightened.',
  }
}

newTalent {
  name = 'Blindness',
  short_name = 'NPC_BLIND',
  display_entity = Entity.new { image = 'talents/blindness.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=6 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.effect_spell,
  info = 'Blinds the target for 11+1d4 turns.',
  message = "@Source@ casts a spell, burning @target@'s eyes.",
  params = {
    eff_id = 'EFF_BLIND', resist_dam = DamageType.BLIND,
    duration = function() return 11 + rng.range(1, 4) end,
    resist_msg = '#Target# is unaffected!',
    save_msg = '#Target# resists the effects!',
  }
}

newTalent {
  name = 'Confusion',
  short_name = 'NPC_CONF',
  display_entity = Entity.new { image = 'talents/confusion.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=7 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.effect_spell,
  info = 'Confuses the target for 3+1d4 turns.',
  message = '@Source@ casts a mesmerizing illusion at @target@.',
  params = {
    eff_id = 'EFF_CONFUSED', resist_dam = DamageType.CONFUSE,
    duration = function() return 3 + rng.range(1, 4) end,
    resist_msg = '#Target# disbelieves the feeble spell.',
  }
}

newTalent {
  name = 'Slowness',
  short_name = 'NPC_SLOW',
  display_entity = Entity.new { image = 'talents/slowness.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=10 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.effect_spell,
  info = 'Slows the target for 3+1d4 turns.',
  message = "@Source@ drains power from @target@'s muscles!",
  params = {
    eff_id = 'EFF_SLOW', resist_flag = 'FREE_ACT', resist_unique = true,
    duration = function() return 3 + rng.range(1, 4) end,
    resist_msg = '#Target# is unaffected!',
    save_msg = '#Target# resists the effects.',
  }
}

newTalent {
  name = 'Paralyse',
  short_name = 'NPC_HOLD',
  display_entity = Entity.new { image = 'talents/paralyze.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=10 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.effect_spell,
  info = 'Paralyzes the target for 3+1d4 turns.',
  message = "@Source@ stares deeply into @target@'s eyes!",
  params = {
    eff_id = 'EFF_PARALYZED', resist_unique = true,
    resist_flags = { 'NO_STUN', 'FREE_ACT' },
    duration = function() return 3 + rng.range(1, 4) end,
    resist_msg = '#Target# is unaffected.',
    save_msg = '#Target# resists the effects.'
  }
}

newTalent {
  name = 'Haste Self',
  short_name = 'NPC_HASTE',
  display_entity = Entity.new { image = 'talents/haste_self.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=50 },
  message = false,
  action = function(self, t)
    spell.logSeen({caster=self}, '#Caster# concentrates on #caster_poss# body.')
    self:setEffect(self.EFF_SPEED, self.level + rng.range(1, 20+self.level), {
      speed = 10,
      merge = function(c, n) return c + rng.range(1, 5) end,
    })
    return true
  end,
  info = function(self, t)
    local tmpl = [[Temporarily increases speed by 10 for %d+1d%d turns.]]
    return tmpl:format(self.level, 20+self.level)
  end,
}

newTalent {
  name = 'Hand of Doom',
  short_name = 'NPC_HAND_DOOM',
  display_entity = Entity.new { image = 'talents/hand_of_doom.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=30, major=true },
  range = 18,
  requires_target = true,
  direct_hit = true,
  action = spell.npc.cause_wounds,
  info = [[Harms the target for 65%-80% of current hp, but never enough to kill outright.]],
  spell_damage = function(e)
    return math.min(((65 + rng.range(1, 25)) * e.life)/100, e.life - 1)
  end,
  curse_chance = { 100, 20 },
  target_resist = function(caster, target)
    return caster.level + rng.range(1, 10) <= target.level + 10 + rng.range(1, 20)
  end,
  message = '@Source@ invokes the Hand of Doom!',
  params = { plyr_hit_msg = 'You feel your life fade away!' },
}

newTalent {
  name = 'Healing',
  short_name = 'NPC_HEAL',
  display_entity = Entity.new { image = 'talents/healing.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=60 },
  message = false,
  action = function(self, t)
    local heal_amt = self.player and rng.dice(8, 5) or self.level*6
    spell.logSeen({caster=self}, '#Caster# concentrates on #caster_poss# wounds.')
    actor:heal(heal_amt, self)
    if not self.player then
      local sfx = actor.life >= actor.max_life and 'completely healed!' or 'healthier.'
      spell.logSeen({caster=self}, '#Caster# looks '..sfx)
      self:removeEffect(self.EFF_AFRAID)
    end
    return true
  end,
  info = function(self, t)
    if self == game.player then return 'Heals the caster for 8d5 hit points.' end
    local tmpl = [[Heals the caster for %d hit points.]]
    return tmpl:format(self.level*6)
  end,
}

newTalent {
  name = 'Phase Door',
  short_name = 'NPC_BLINK',
  display_entity = Entity.new { image = 'talents/phase_door.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=2 },
  requires_target = true,
  direct_hit = true,
  message = '@Source@ blinks away.',
  action = function(self, t)
    spell.teleport(self, 10)
    return true
  end,
  info = 'Teleports the caster up to 10 grids.',
}

newTalent {
  name = 'Teleport',
  short_name = 'NPC_TPORT',
  display_entity = Entity.new { image = 'talents/teleport.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=10 },
  requires_target = true,
  direct_hit = true,
  message = '@Source@ teleports away.',
  action = function(self, t)
    local r = self == game.player and self.level*5 or 45
    spell.teleport(self, r)
    return true
  end,
  info = function(self, t)
    local r = self == game.player and self.level*5 or 45
    local tmpl = [[Teleports the caster up to %d grids.]]
    return tmpl:format(r)
  end,
}

newTalent {
  name = 'Teleport To',
  short_name = 'NPC_TELE_TO',
  display_entity = Entity.new { image = 'talents/teleport_to.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  action = function(self, t)
  end,
  info = 'Currently unimplemented.  We Apologize for the Inconvenience.™',
}

newTalent {
  name = 'Teleport Away',
  short_name = 'NPC_TELE_AWAY',
  display_entity = Entity.new { image = 'talents/teleport_away.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20 },
  range = 18,
  requires_target = true,
  direct_hit = true,
  message = '@Source@ teleports @target@ away.',
  action = function(self, t)
    local x, y, target = spell.npc.get_spell_target(self, t)
    if not target then return end
    spell.teleport(target, target == game.player and 100 or 45)
    return true
  end,
  info = function(self, t)
    local tmpl = [[Teleports the target up to %d grids away.]]
    return tmpl:format(self == game.player and 100 or 45)
  end,
}

newTalent {
  name = 'Teleport Level',
  short_name = 'NPC_TELE_LEVEL',
  display_entity = Entity.new { image = 'talents/teleport_level.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=20, major=true },
  message = "@Source@ gestures at @target@'s feet.",
  action = function(self, t)
    if self.player then
      spell.player_teleport_level()
      return true
    end
    local x, y, target = spell.npc.get_spell_target(self, t)
    if not target or not target.player then return end
    spell.player_teleport_level()
    return true
  end,
  info = 'Teleport up or down a level randomly.',
}

newTalent {
  name = 'Darkness',
  short_name = 'NPC_DARKNESS',
  display_entity = Entity.new { image = 'talents/darkness.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=3 },
  requires_target = true,
  message = '@Source@ gestures in shadow.',
  action = function(self, t)
    if self.player then
      spell.ball_spell(self, t, DamageType.DARK_WEAK, 0, 3, self.x, self.y)
      spell.unlite_room(self.x, self.y)
    else
      spell.ball_spell(self, t, DamageType.DARK_WEAK, 0, 3)
    end
    return true
  end,
  info = 'Plunge your surroundings into darkness.',
}

newTalent {
  name = 'Create Traps',
  short_name = 'NPC_TRAPS',
  display_entity = Entity.new { image = 'talents/create_traps.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=10, major=true },
  requires_target = true,
  action = spell.npc.ball_spell,
  ball_damage = function(e) return 1 end,
  message = '@Source@ casts a spell and cackles evilly.',
  params = { r = 2, type = DamageType.MAKE_TRAP },
  info = 'Creates traps in a radius-1 targed area.',
}

newTalent {
  name = 'Raise the Dead',
  short_name = 'NPC_ANIM_DEAD',
  display_entity = Entity.new { image = 'talents/raise_the_dead.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  player = { mana=400, major=true },
  action = function(self, t)
  end,
  info = 'Currently unimplemented.  We Apologize for the Inconvenience.™',
}

newTalent {
  name = 'Cause Amnesia',
  short_name = 'NPC_FORGET',
  display_entity = Entity.new { image = 'talents/cause_amnesia.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  requires_target = true,
  action = function(self, t)
    local x, y, target = spell.npc.get_spell_target(self, t)
    if target and target.player then
      game.logPlayer(target, '%s tries to blank your mind.', self:getName{def_art=true, check_seen=true}:capitalize())
      if rng.percent(target.saving_throw or 0) then
	game.logPlayer(target, 'You resist the effects!')
      else
	game.logPlayer(target, 'Your memories fade away.')
	spell.player_amnesia()
      end
    end
    return true
  end,
  info = [[Scrambles the target's memory of the level, including traps and objects on the floor.]],
}

newTalent {
  name = 'Mind Blast',
  short_name = 'NPC_MIND_BLAST',
  display_entity = Entity.new { image = 'talents/mind_blast.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  requires_target = true,
  action = function(self, t)
    local src_name = self:getName{def_art=true, check_seen=true}
    local x, y, target = spell.npc.get_spell_target(self, t)
    if target then
      local tgt_name = target:getName{def_art=true, check_seen=true}
      if target.player then
	if target:canSee(self) then
	  game.logPlayer(target, 'You feel something focusing on your mind.')
	else
	  game.logPlayer(target, '%s gazes deep into your eyes.', src_name:capitalize())
	end
	if rng.percent(target.saving_throw or 0) then
	  game.logPlayer(target, 'You resist the effects!')
	else
	  game.logPlayer(target, 'Your mind is blasted by psionic energy.')
	  if not target:resistDam(DamType.CONFUSION) then
	    target:setEffect(target.EFF_CONFUSED, rng.range(4, 7), {merge='combine'})
	  end
	  if not target:resistDam(DamType.HALLUCINATION) and rng.chance(3) then
	    target:setEffect(target.EFF_HALLUCINATION, rng.range(0, 249) + 150, {merge='combine'})
	  end
	  target:takeSanityHit(rng.dice(3, 8), self)
	end
      else
	game.logSeen(self, '%s gazes intensely at %s.', src_name:capitalize(), tgt_name)
	if target.unique or target:resistDam(DamageType.CONFUSION, {or_immune=true, learn=true}) or target.level > rng.range(1, math.min(self.level, 11)) then
	  game.logSeen(target, '% is unaffected!', tgt_name:capitalize())
	else
	  game.logPlayer(target, '%s is blasted by psionic energy.', tgt_name:capitalize())
	  target:setEffect(target.EFF_CONFUSED, rng.range(4, 7), {merge='combine'})
	  target:takeHit(rng.dice(8, 8), self)
	end
      end
    end
    return true
  end,
  info = [[Blasts the target's mind with psionic energy, inducing confusion, possible hallucination and sanity damage.]],
}

newTalent {
  name = 'Brain Smash',
  short_name = 'NPC_BRAIN_SMASH',
  display_entity = Entity.new { image = 'talents/brain_smash.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  requires_target = true,
  action = function(self, t)
    local src_name = self:getName{def_art=true, check_seen=true}
    local x, y, target = spell.npc.get_spell_target(self, t)
    if target then
      local tgt_name = target:getName{def_art=true, check_seen=true}
      if target.player then
	if target:canSee(self) then
	  game.logPlayer(target, 'You feel something focusing on your mind.')
	else
	  game.logPlayer(target, '%s gazes deep into your eyes.', src_name:capitalize())
	end
	if rng.percent(target.saving_throw or 0) then
	  game.logPlayer(target, 'You resist the effects!')
	else
	  game.logPlayer(target, 'Your mind is blasted by psionic energy.')
	  target:takeSanityHit(rng.dice(12, 15), self)
	  if not target:resistDam(DamType.BLIND) then
	    target:setEffect(target.EFF_BLIND, rng.range(8, 15), {merge='combine'})
	  end
	  if not target:resistDam(DamType.CONFUSION) then
	    target:setEffect(target.EFF_CONFUSED, rng.range(4, 7), {merge='combine'})
	  end
	  if not target:has('FREE_ACT') then
	    target:setEffect(target.EFF_PARALYZED, rng.range(4, 7), {merge='combine'})
	  end
	  while not rng.percent(target.saving_throw) do
	    target:bumpStatDown('INT')
	  end
	  while not rng.percent(target.saving_throw) do
	    target:bumpStatDown('WIS')
	  end
	  if not target:resistDam(DamType.HALLUCINATION) then
	    target:setEffect(target.EFF_HALLUCINATION, rng.range(0, 249) + 150, {merge='combine'})
	  end
	end
      else
	game.logSeen(self, '%s gazes intensely at %s.', src_name:capitalize(), tgt_name)
	if target.unique or target:resistDam(DamageType.CONFUSION, {or_immune=true, learn=true}) or target.level > rng.range(1, math.min(self.level, 11)) then
	  game.logSeen(target, '% is unaffected!', tgt_name:capitalize())
	else
	  game.logPlayer(target, '%s is blasted by psionic energy.', tgt_name:capitalize())
	  target:setEffect(target.EFF_CONFUSED, rng.range(4, 7), {merge='combine'})
	  target.speed = math.max(-50, target.speed - rng.range(4, 7))
	  target:computeGlobalSpeed()
	  target:setEffect(target.EFF_STUN, rng.range(4, 7), {merge='combine'})
	  target:takeHit(rng.dice(12, 15), self)
	end
      end
    end
    return true
  end,
  info = [[Smash the target's brain with psionic energy, inducing sanity damage, blindness, confusion, paralysis, hallucination, and possible loss of Intelligence and Wisdom.]],
}

newTalent {
  name = 'Drain Mana',
  short_name = 'NPC_DRAIN_MANA',
  display_entity = Entity.new { image = 'talents/drain_mana.png' },
  type = 'npc/spell',
  mode = 'activated',
  is_spell = true,
  points = 0,
  requires_target = true,
  action = function(self, t)
    local x, y, target = spell.npc.get_spell_target(self, t)
    if not target then return end
    local src_name = self:getName{def_art=true, check_seen=true}
    local tgt_name = target:getName{def_art=true, check_seen=true}
    local r1 = 1 + math.floor(rng.range(1, target.level/2))
    game.logSeen(self, '%s draws psyhic energy from %s.', src_name:capitalize(), tgt_name)
    -- The T2 code assumes an NPC has mana if it has any NPC talents.
    local has_mana = (target.player and target.max_mana) or (not target.player and target.talents)
    if has_mana then
      if target.player and target.max_mana then
	r1 = math.max(r1, target.mana)
	target:incMana(-r1)
      end
      local was_hurt = self.life < self.max_life
      self:heal(6*r1)
      if was_hurt then
	game.logSeen(self, '%s appears healthier.', src_name:capitalize())
      end
    end
    return true
  end,
  info = 'Drains mana from the target, healing the caster.',
}
