-- $Id: zone.lua 2508 2016-09-14 02:02:13Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Zone file for The Helcaraxë.

local DamageType = require('engine.DamageType')

return {
  name = 'The Helcaraxë',
  display_abbr = 'Ice',
  level_scheme = 'fixed',
  max_level = 21,
  level_range = { 20, 40 },
  width = 198,
  height = 66,
  ambient_damage = {
    damage = { 1, 4 },
    frequency = 15,
    damtype = DamageType.COLD,
    kill_text = [[terminal exposure to cold]],
  },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { ICE={0,90}, DIRT={70,0}, SHALLOW_WATER={30,10} } },
      fill = { weighted = { ICE_WALL={0,100}, WALL={100,0} } },
      outer_wall = 'WALL',
      inner_wall = 'WALL',
      fill_step = 2,
      up = 'UP',
      down = 'DOWN',
      flags = {
	DOUBLE = true,
	WATER_RIVER = true,
	CAVERN = true,
	NO_STREAMERS = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=100, mode='and', resist_dam={ DamageType.COLD } },
      },
      guardian = 'THE_WHITE_BALROG',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
    [16] = {
      no_level_connectivity = true,
      level_name = [[Galleon]],
      enter_desc = [[A ship of antique design lies jammed in the ice here.]],
      generator = {
	map = {
	  class='mod.class.generator.map.Static',
	  map = 'zones/galleon',
	  one_time_level = 'galleon',
	  flags = { DESC=true, NO_GENO=true, NO_NEW_MONSTER=true, SPECIAL=true, NO_STAIR=true, ASK_LEAVE=true, NO_TELEPORT=true },
	},
	actor = {
	  init_pc = table.NIL_MERGE,
	  respawn_in = table.NIL_MERGE,
	},
      },
    },
  },
}
