-- $Id: zone.lua 1900 2015-11-20 00:30:52Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2015 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Zone file for a lost temple.

-- This will go badly if we don't have a relic quest; we set up just enough
-- scaffolding that the change-zone debug dialog won't choke.
local q = game.player:hasQuest('relic') or { top_level=0 }
local god_d = require('engine.Birther'):getBirthDescriptor('god', game.player.descriptor.god)
local gqzd = god_d.quest and god_d.quest.zone or {}

return {
  name = 'Lost Temple',
  display_abbr = 'LTm',
  level_scheme = 'fixed',
  max_level = 5,
  level_range = { q.top_level, q.top_level + 4 },
  width = 198,
  height = 66,

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = gqzd.floor,
      fill = gqzd.fill,
      outer_wall = gqzd.outer_wall,
      inner_wall = gqzd.inner_wall,
      fill_step = gqzd.fill_step,
      up = 'UP',
      down = 'DOWN',
      flags = gqzd.flags,
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = gqzd.init_npc,
      respawn_in = gqzd.respawn_in,
      filters = gqzd.actor_filters,
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = gqzd.obj_weights,
    },
    -- trap = {},
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
  },

  actor_level_range = gqzd.actor_level_range,
  actor_adjust_level = function(zone, level, e)
    local lev = zone:level_adjust_level(level, 'actor')
    local min_lev, max_lev = zone.actor_level_range(lev)
    return rng.range(min_lev, max_lev)
  end,

  post_process = function(level, zone)
    local q = game.player:hasQuest('relic')
    if level.level == q.relic_level then
      -- Place the relic on the level.
      print('[ZONE] attempting to place relic')
      local Map = require 'mod.class.Map'
      local map = level.map
      local x, y = rng.range(0, map.w-1), rng.range(0, map.h-1)
      local tries = 0
      while (not map:checkEntity(x, y, Map.TERRAIN, 'floor') or
	     not map:checkEntity(x, y, Map.TERRAIN, 'permanent') or
	     map(x, y, Map.TRAP)) and tries < 10000
      do
	x, y = rng.range(0, map.w-1), rng.range(0, map.h-1)
	tries = tries + 1
      end
      if tries < 10000 then
	local god_d = require('engine.Birther'):getBirthDescriptor('god', game.player.descriptor.god)
	local o = zone:makeEntityByName(level, 'object', god_d.quest.relic)
	if o then
	  print(('[ZONE] placing relic at %d,%d'):format(x, y))
	  zone:addEntity(level, o, 'object', x, y)
	  -- TODO Make this immune to automatizer
	  o:setFoundHere({ type='floor' }, zone, level)
	else
	  error('[ZONE] unable to create '..god_d.quest.relic)
	end
      else
	error('[ZONE] could not find location to place relic')
      end
    end
  end,
}
