-- $Id: zone.lua 2508 2016-09-14 02:02:13Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Zone file for The Small Water Cave.

local DamageType = require('engine.DamageType')

return {
  name = 'The Small Water Cave',
  display_abbr = 'SWC',
  level_scheme = 'fixed',
  max_level = 3,
  level_range = { 32, 34 },
  width = 198,
  height = 66,
  ambient_damage = {
    damage = { 1, 1 },
    frequency = 20,
    damtype = DamageType.ACID,
    kill_text = [[acidic waters]],
  },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { 'SHALLOW_WATER' },
      fill = { 'MOUNTAIN_CHAIN' },
      outer_wall = 'WALL',
      inner_wall = 'WALL',
      fill_step = 0,
      up = 'UP',
      down = 'DOWN',
      flags = {
	NO_RECALL = true,
	NO_UP = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=10, mode='any' },
	{ prob=10, mode='or', flags={ 'AQUATIC' } },
	{ prob=40, mode='and', talents={ 'T_BA_WATE' }, resist_dam={ DamageType.COLD } },
	{ prob=40, mode='or', resist_dam={ DamageType.COLD } },
      },
      guardian = 'THE_WATCHER_IN_THE_WATER',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=10, combat=10, magic=30, tools=30, junk=20 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_MORIA',
	  -- up_lev = 11
	},
      },
    },
  },
}
