-- $Id: zone.lua 2508 2016-09-14 02:02:13Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2016 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- Zone file for Submerged Ruins.

local DamageType = require('engine.DamageType')

return {
  name = 'Submerged Ruins',
  display_abbr = 'Num',
  level_scheme = 'fixed',
  max_level = 16,
  level_range = { 35, 50 },
  width = 198,
  height = 66,
  ambient_damage = {
    damage = { 1, 1 },
    frequency = 1,
    damtype = DamageType.ACID,
    kill_text = [[acidic waters]],
  },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { SHALLOW_WATER=95, DEEP_WATER=5 } },
      fill = { weighted = { DEEP_WATER={80,60}, SHALLOW_WATER={10,0}, WALL={10,40} } },
      outer_wall = 'WALL',
      inner_wall = 'DEEP_WATER',
      fill_step = 4,
      up = 'UP',
      down = 'DOWN',
      flags = {
	NO_STREAMERS = true,
	WATER_BREATH = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=20, mode='any' },
	{ prob=80, mode='or', flags={ 'AQUATIC', 'CAN_SWIM', 'CAN_FLY' } },
      },
      guardian = 'AR_PHARAZON_THE_GOLDEN',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=30, combat=30, magic=10, tools=10, junk=20 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
  },
}
