-- $Id: HotkeysIconsDisplay.lua 2134 2016-01-26 02:16:32Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- A modified version of engine.HotkeysIconsDisplay that uses our
-- extensible HotkeysIcons infrastructure to support hotkey types that are
-- not supported by the original.

require 'engine.class'
local Super = require 'engine.HotkeysIconsDisplay'
local Icons = require 'mod.class.HotkeysIcons'

module(..., package.seeall, class.inherit(Super))

-- Copied from the parent, which declares it as local so we can't access it
-- directly.
local page_to_hotkey = {"", "SECOND_", "THIRD_", "FOURTH_", "FIFTH_"}

-- Copied from engine.HotkeysIconsDisplay:display() and modified to use our
-- extensible HotkeysIcons infrastructure.
function _M:display()
  local a = self.actor
  if not a or not a.changed then return self.surface end

  local bpage = a.hotkey_page
  local spage = bpage

  self.surface:erase(self.bgcolor[1], self.bgcolor[2], self.bgcolor[3])
  if self.bg_surface then self.surface:merge(self.bg_surface, 0, 0) end

  local orient = self.orient or "down"
  local x = 0
  local y = 0
  local col, row = 0, 0
  self.dragclics = {}
  self.clics = {}
  self.items = {}
  local w, h = self.frames.w, self.frames.h

  for page = bpage, #page_to_hotkey do
    for i = 1, 12 do
      local ts = nil
      local bi = i
      local j = i + (12 * (page - 1))
      if a.hotkey[j] and Icons:hasType(a.hotkey[j][1]) then
	ts = { a.hotkey[j][2], j, a.hotkey[j][1], i, page, i + (12 * (page - bpage)) }
      end

      x = self.frames.w * col
      y = self.frames.h * row
      self.dragclics[j] = {x,y,w,h}

      if ts then
	local s
	local i = ts[2]
	local lpage = ts[5]
	local display_entity, frame, color, angle, txt = Icons:display(ts, a)
	frame, angle = (frame or 'ok'), (angle or 0)

	self.font:setStyle("bold")
	local ks = game.key:formatKeyString(game.key:findBoundKeys("HOTKEY_"..page_to_hotkey[page]..bi))
	-- Antique white on white doesn't show up well; for now, change the
	-- color of the hotkey text.
	--local key = self.font:draw(ks, self.font:size(ks), colors.ANTIQUE_WHITE.r, colors.ANTIQUE_WHITE.g, colors.ANTIQUE_WHITE.b, true)[1]
	local key = self.font:draw(ks, self.font:size(ks), colors.LIGHT_RED.r, colors.LIGHT_RED.g, colors.LIGHT_RED.b, true)[1]
	self.font:setStyle("normal")

	local gtxt = nil
	if txt then
	  gtxt = self.fontbig:draw(txt, w, colors.WHITE.r, colors.WHITE.g, colors.WHITE.b, true)[1]
	  gtxt.fw, gtxt.fh = self.fontbig:size(txt)
	end

	self.items[#self.items+1] = {i=i, x=x, y=y, e=display_entity or self.default_entity, color=color, angle=angle, key=key, gtxt=gtxt, frame=frame, pagesel=lpage==spage}
	self.clics[i] = {x,y,w,h}
      else
	local i = i + (12 * (page - 1))
	local angle = 0
	local color = {190,190,190}
	local frame = "disabled"

	self.font:setStyle("bold")
	local ks = game.key:formatKeyString(game.key:findBoundKeys("HOTKEY_"..page_to_hotkey[page]..bi))
	-- As above.
	--local key = self.font:draw(ks, self.font:size(ks), colors.ANTIQUE_WHITE.r, colors.ANTIQUE_WHITE.g, colors.ANTIQUE_WHITE.b, true)[1]
	local key = self.font:draw(ks, self.font:size(ks), colors.LIGHT_RED.r, colors.LIGHT_RED.g, colors.LIGHT_RED.b, true)[1]
	self.font:setStyle("normal")

	self.items[#self.items+1] = {show_on_drag=true, i=i, x=x, y=y, e=nil, color=color, angle=angle, key=key, gtxt=nil, frame=frame}
	self.clics[i] = {x,y,w,h, fake=true}
      end

      if orient == "down" or orient == "up" then
	col = col + 1
	if col >= self.max_cols then
	  col = 0
	  row = row + 1
	  if row >= self.max_rows then return end
	end
      elseif orient == "left" or orient == "right" then
	row = row + 1
	if row >= self.max_rows then
	  row = 0
	  col = col + 1
	  if col >= self.max_cols then return end
	end
      end
    end
  end
end

-- Copied from engine.HotkeysIconsDisplay:onMouse() and modified to use our
-- extensible HotkeysIcons infrastructure.
function _M:onMouse(button, mx, my, click, on_over, on_click)
  local orient = self.orient or "down"
  mx, my = mx - self.display_x, my - self.display_y
  local a = self.actor

  if button == "wheelup" and click then
    a:prevHotkeyPage()
    return
  elseif button == "wheeldown" and click then
    a:nextHotkeyPage()
    return
  elseif button == "drag-end" then
    local drag = game.mouse.dragged.payload
    if Icons:hasType(drag.kind) then
      for i, zone in pairs(self.dragclics) do
	if mx >= zone[1] and mx < zone[1] + zone[3] and my >= zone[2] and my < zone[2] + zone[4] then
	  local old = self.actor.hotkey[i]

	  if i <= #page_to_hotkey * 12 then -- Only add this hotkey if we support a valid page for it.
	    self.actor.hotkey[i] = {drag.kind, drag.id}

	    if drag.source_hotkey_slot then
	      self.actor.hotkey[drag.source_hotkey_slot] = old
	    end

	    -- Update the quickhotkeys table immediately rather than waiting for a save.
	    if self.actor.save_hotkeys then
	      engine.interface.PlayerHotkeys:updateQuickHotkey(self.actor, i)
	      engine.interface.PlayerHotkeys:updateQuickHotkey(self.actor, drag.source_hotkey_slot)
	    end
	  end
	  game.mouse:usedDrag()
	  self.actor.changed = true
	  break
	end
      end
    end
  end

  for i, zone in pairs(self.clics) do
    if mx >= zone[1] and mx < zone[1] + zone[3] and my >= zone[2] and my < zone[2] + zone[4] then
      if on_click and click and not zone.fake then
	if on_click(i, a.hotkey[i]) then click = false end
      end
      local oldsel = self.cur_sel
      self.cur_sel = i
      if button == "left" and not zone.fake then
	if click then
	  a:activateHotkey(i)
	else
	  local s = Icons:dragSurface(a.hotkey[i], a)
	  game.mouse:startDrag(mx, my, s, {kind=a.hotkey[i][1], id=a.hotkey[i][2], source_hotkey_slot=i}, function(drag, used) if not used then self.actor.hotkey[i] = nil self.actor.changed = true end end)
	end
      elseif button == "right" and click and not zone.fake then
	a.hotkey[i] = nil
	a.changed = true
      else
	a.changed = true
	if on_over and self.cur_sel ~= oldsel and not zone.fake then
	  local text = Icons:tooltipText(a.hotkey[i], a) or ''
	  on_over(text)
	end
      end
      return
    end
  end
  self.cur_sel = nil
end
