-- $Id: ShimmerOverlay.lua 994 2014-08-30 15:53:10Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'
local Entity = require 'engine.Entity'

module(..., package.seeall, class.inherit(Entity))

function _M:init(colors)
  Entity.init(self, { display='', alpha=128 })
  self.bg_colors = table.clone(colors, true)
  self.tiles = {}
  self.grids = {}
  self.last_update = 0
end

function _M:tileForXY(x, y)
  if #self.tiles == 0 then self:preFetchTiles() end

  -- Sneaky Hack(TM):  Change tiles when the game turn changes, so that it
  -- won't flicker while the player's doing nothing.
  if self.last_update ~= game.turn then
    self.grids = {}
    self.last_update = game.turn
  end

  self.grids[x] = self.grids[x] or {}
  self.grids[x][y] = self.grids[x][y] or rng.table(self.tiles)
  return self.grids[x][y]
end

function _M:preFetchTiles()
  if #self.tiles == 0 then
    for i, bg in ipairs(self.bg_colors) do
      self.tiles[i] = game.level.map.tilesEffects:get('', 0, 0, 0, bg.r, bg.g, bg.b, nil, 128)
    end
  end
end
