-- $Id: T2DungeonRooms.lua 2438 2016-07-25 00:58:00Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'

module(..., package.seeall, class.make)

_M.room_types = {}
_M.vault_types = {}

-- Load room definitions.
-- Static
function _M:loadDefinition(file, env)
  env = env or setmetatable({
    Map = require 'mod.class.Map',
    newRoom = function(t) self:newRoom(t) end,
    newVault = function(t) self:newVault(t) end,
    load = function(f) self:loadDefinition(f, env) end,
  }, { __index=_G })
  local f, err = loadfile(file)
  if not f then
    if err then error(err) end
    return
  end
  setfenv(f, env)
  f()
end

function _M:newRoom(t)
  assert(t.id, 'no room definition id')
  assert(not self.room_types[t.id], 'room type "'..t.id..'" already defined')
  assert(t.min_depth or t.no_random_gen, 'no room definition min_depth field')
  assert(t.build or t.build_special, 'no room definition build() function')
  self.room_types[t.id] = t
  table.insert(self.room_types, t)
end

function _M:newVault(t)
  assert(t.id, 'no vault definition id')
  assert(t.type, 'no vault definition type')
  self.vault_types[t.type] = self.vault_types[t.type] or {}
  assert(not self.vault_types[t.type][t.id], 'vault type "'..t.id..'" already defined')
  assert(t.name, 'no vault definition name')
  assert(t.map, 'no vault definition map file')
  assert(t.width, 'no vault definition width')
  assert(t.height, 'no vault definition height')
  assert(t.rating, 'no vault definition rating')
  self.vault_types[t.type][t.id] = t
  table.insert(self.vault_types[t.type], t)
end
