-- $Id: ShowEquipInven.lua 2003 2015-12-25 01:25:13Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'
local ShowEquipInven = require 'engine.dialogs.ShowEquipInven'

module(..., package.seeall, class.inherit(ShowEquipInven))

function _M:init(title, actor, filter, action, on_select)
  -- Show a tooltip when selecting an item.
  local tooltip_cb = function(item)
    if item.last_display_x and item.object then
      local x
      if self.focus_ui and self.focus_ui.ui == self.c_inven then
	x = item.last_display_x - game.tooltip.w
      elseif self.focus_ui and self.focus_ui.ui == self.c_equip then
	x = item.last_display_x + self.c_equip.w
      end
      game:tooltipDisplayAtMap(x or item.last_display_x, item.last_display_y, item.object:getDesc({do_color=true}, actor:getInven(item.object:wornInven())))
    elseif item.last_display_x and item.data and item.data.desc then
      game:tooltipDisplayAtMap(item.last_display_x, item.last_display_y, item.data.desc, {up=true})
    end
    if on_select then on_select(item) end
  end

  ShowEquipInven.init(self, title, actor, filter, action, tooltip_cb)

  -- Widen the 'char' column a bit to stop the @#$%ing annoying wiggling.
  self.c_inven:setColumns {
    {name="", width={26,"fixed"}, display_prop="char", sort="id"},
    {name="", width={24,"fixed"}, display_prop="object", sort="sortname", direct_draw=function(item, x, y) if item.object then item.object:toScreen(nil, x+4, y, 16, 16) end end},
    {name="Inventory", width=72, display_prop="name", sort="sortname"},
    {name="Category", width=20, display_prop="cat", sort="cat"},
    {name="Enc.", width=8, display_prop="encumberance", sort="encumberance"},
  }
  self.c_equip:setColumns {
    {name="", width={26,"fixed"}, display_prop="char"},
    {name="", width={8+16,"fixed"}, display_prop="object", direct_draw=function(item, x, y) if item.object then item.object:toScreen(nil, x+4, y, 16, 16) end end},
    {name="Equipment", width=72, display_prop="name"},
    {name="Category", width=20, display_prop="cat"},
    {name="Enc.", width=8, display_prop="encumberance"},
  }
end

function _M:generateList(no_update)
  ShowEquipInven.generateList(self, no_update)

  -- Slight Hack(TM):  Hide any equipment slots that are supposed to be
  -- hidden.
  local list = { chars={} }
  for _, l in ipairs(self.equip_list) do
    local def = self.actor.inven_def[l.inven]
    local hide = util.getval(def.infos and def.infos.hide, self.actor)
    if not hide then
      list[#list+1] = l
      l.id = #list
      if l.char ~= '' then list.chars[l.char] = #list end
    end
  end
  self.equip_list = list
  if not no_update then self.c_equip:setList(self.equip_list) end

  -- Slight Hack(TM):  If the generated equipment or inventory list is
  -- empty, it won't get properly pushed into the corresponding ListColumns
  -- list, and the 'chars' table we generated will be missing, causing an
  -- error when the user types anything.
  if not no_update then
    if #self.equip_list == 0 then self.c_equip.list.chars = {} end
    if #self.inven_list == 0 then self.c_inven.list.chars = {} end
  end
end

function _M:inventoryLiveUpdate()
  self:generateList()
end
