-- $Id: ShowEquipInvenFloor.lua 2379 2016-05-01 23:30:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'
local ShowEquipInven = require 'engine.dialogs.ShowEquipInven'

module(..., package.seeall, class.inherit(ShowEquipInven))

function _M:init(title, actor, filter, action, on_select, t)
  t = t or {}

  -- Show a tooltip when selecting an item.
  local tooltip_cb = function(item)
    if item.last_display_x and item.object then
      local x
      if self.focus_ui and self.focus_ui.ui == self.c_inven then
	x = item.last_display_x - game.tooltip.w
      elseif self.focus_ui and self.focus_ui.ui == self.c_equip then
	x = item.last_display_x + self.c_equip.w
      end
      game:tooltipDisplayAtMap(x or item.last_display_x, item.last_display_y, item.object:getDesc({do_color=true}, actor:getInven(item.object:wornInven())))
    elseif item.last_display_x and item.data and item.data.desc then
      game:tooltipDisplayAtMap(item.last_display_x, item.last_display_y, item.data.desc, {up=true})
    end
    if on_select then on_select(item) end
  end

  ShowEquipInven.init(self, title, actor, filter, action, tooltip_cb)

  if t.allow_drag then
    self.c_equip.on_drag = function(item) self:startDrag(item) end
    self.c_equip.on_drag_end = function(item) self:endDrag(item) end
    self.c_inven.on_drag = function(item) self:startDrag(item) end
    self.c_inven.on_drag_end = function(item) self:endDrag(item) end
  end

  -- Widen the 'char' column a bit to stop the @#$%ing annoying wiggling.
  self.c_inven:setColumns {
    {name="", width={26,"fixed"}, display_prop="char", sort="id"},
    {name="", width={24,"fixed"}, display_prop="object", sort="sortname", direct_draw=function(item, x, y) if item.object then item.object:toScreen(nil, x+4, y, 16, 16) end end},
    {name="Inventory", width=72, display_prop="name", sort="sortname"},
    {name="Category", width=20, display_prop="cat", sort="cat"},
    {name="Enc.", width=8, display_prop="encumberance", sort="encumberance"},
  }
  self.c_equip:setColumns {
    {name="", width={26,"fixed"}, display_prop="char"},
    {name="", width={8+16,"fixed"}, display_prop="object", direct_draw=function(item, x, y) if item.object then item.object:toScreen(nil, x+4, y, 16, 16) end end},
    {name="Equipment", width=72, display_prop="name"},
    {name="Category", width=20, display_prop="cat"},
    {name="Enc.", width=8, display_prop="encumberance"},
  }
end

function _M:use(item, button, event)
  if item and item.object then
    if self.action(item.object, item.inven, item.floor, item.item, button, event) then
      game:unregisterDialog(self)
    end
  end
end

function _M:generateList(no_update)
  -- Equipment list.
  self.equip_list = { chars={} }
  local list = self.equip_list
  local i = 1
  self.max_h = 0
  for inven_id =  1, #self.actor.inven_def do
    local def = self.actor.inven_def[inven_id]
    if self.actor.inven[inven_id] and (def.is_worn or def.is_shown_equip) and not util.getval(def.infos and def.infos.hide, self.actor) then
      list[#list+1]  =  {
	id = #list+1,
	char = "",
	name = ('#{bold}#' .. self.actor.inven_def[inven_id].name .. '#{normal}#'):toTString(),
	color = {0x90, 0x90, 0x90},
	inven = inven_id,
	cat = "",
	encumberance = "",
	desc = self.actor.inven_def[inven_id].description,
      }
      self.max_h = math.max(self.max_h, #self.actor.inven_def[inven_id].description:splitLines(self.iw - 10, self.font))

      for item, o in ipairs(self.actor.inven[inven_id]) do
	if not self.filter or self.filter(o) then
	  local char = self:makeKeyChar(i)

	  local enc = 0
	  o:forAllStack(function(o) enc=enc+o.encumber end)

	  list[#list+1]  =  {
	    id = #list+1,
	    char = char,
	    name = o:getName(),
	    sortname = o:getName():toString():removeColorCodes(),
	    color = o:getDisplayColor(),
	    object = o,
	    inven = inven_id,
	    item = item,
	    cat = o.subtype,
	    encumberance = enc,
	    desc = o:getDesc(),
	  }
	  list.chars[char] = #list
	  i = i + 1
	end
      end
    end
  end
  self.equip_list = list

  -- Inventory/floor list.
  self.inven_list = { chars={} }
  local list = self.inven_list
  local x, y = self.actor.x, self.actor.y
  local inven = self.actor:getInven('INVEN')

  local do_inv, do_floor = false, false
  if self.filter then
    for _, o in ipairs(inven or {}) do
      if self.filter(o) then
	do_inv = true
	break
      end
    end
    if x and y then
      for i = 1, game.level.map:getObjectTotal(x, y) do
	if self.filter(game.level.map:getObject(x, y, i)) then
	  do_floor = true
	  break
	end
      end
    end
  else
    do_inv = inven and #inven > 0
    do_floor = x and y and game.level.map:getObjectTotal(x, y) > 0
  end

  local ch = 1
  if do_inv then
    list[#list+1] = {
      id = #list + 1,
      char = '',
      name = ('#{bold}#In inventory#{normal}#'):toTString(),
      sortname = '1:',
      color = {0x90, 0x90, 0x90},
      cat = '',
      sort_cat = '1:',
      encumberance = '',
      sort_enc = '1:',
      desc = 'Items in your inventory',
    }
    for item, o in ipairs(inven or {}) do
      if not self.filter or self.filter(o) then
	local char = self:makeKeyChar(ch)

	local enc = 0
	o:forAllStack(function(o) enc=enc+o.encumber end)

	list[#list+1] = {
	  id = #list + 1,
	  char = char,
	  name = o:getName(),
	  sortname = '1:' .. o:getName():toString():removeColorCodes(),
	  color = o:getDisplayColor(),
	  object = o,
	  inven = self.actor.INVEN_INVEN,
	  item = item,
	  cat = o.subtype,
	  sort_cat = '1:' .. (o.subtype or ''),
	  encumberance = enc,
	  sort_enc = ('1:%d'):format(enc),
	  desc = o:getDesc(),
	}
	list.chars[char] = #list
	ch = ch + 1
      end
    end
  end

  if do_floor then
    list[#list+1] = {
      id = #list + 1,
      char = '',
      name = ('#{bold}#On floor#{normal}#'):toTString(),
      sortname = '2:',
      color = {0x90, 0x90, 0x90},
      floor = true,
      cat = '',
      sort_cat = '2:',
      encumberance = '',
      sort_enc = '2:',
      desc = 'Items on the floor at your feet',
    }
    for i = 1, game.level.map:getObjectTotal(x, y) do
      local o = game.level.map:getObject(x, y, i)
      if not self.filter or self.filter(o) then
	local char = self:makeKeyChar(ch)

	local enc = 0
	o:forAllStack(function(o) enc=enc+o.encumber end)

	list[#list+1] = {
	  id = #list + 1,
	  char = char,
	  name = o:getName(),
	  sortname = '2:' .. o:getName():toString():removeColorCodes(),
	  color = o:getDisplayColor(),
	  object = o,
	  item = i,
	  floor = true,
	  cat = o.subtype,
	  sort_cat = '2:' .. (o.subtype or ''),
	  encumberance = enc,
	  sort_enc = ('2:%d'):format(enc),
	  desc = o:getDesc(),
	}
	list.chars[char] = #list
	ch = ch + 1
      end
    end
  end

  if not no_update then
    self.c_inven:setList(self.inven_list)
    self.c_equip:setList(self.equip_list)
  end
end

function _M:startDrag(item)
  if item and item.object then
    local s = item.object:getEntityFinalSurface(nil, 64, 64)
    local x, y = core.mouse.get()
    local payload = {
      kind = 'inventory',
      item_idx = item.item,
      inven = item.inven,
      floor = item.floor,
      object = item.object,
      id = item.object:getName{force_id=true, no_count=true},
    }
    local function done_cb(drag, used)
      if not used then
	local x, y = core.mouse.get()
	game.mouse:receiveMouse('drag-end', x, y, true, nil, {drag=drag})
      end
    end
    game.mouse:startDrag(x, y, s, payload, done_cb)
  end
end

function _M:endDrag(item)
  local drag = game.mouse.dragged.payload
  if drag.kind ~= 'inventory' then return end
  local from_inv = drag.inven and self.actor:getInven(drag.inven)
  local to_inv = item.inven and self.actor:getInven(item.inven)

  local changed = false
  if item.floor and not drag.floor then
    -- Drag from equip or inventory to floor; drop.
    self.actor:doDrop(drag.inven, drag.item_idx)
    changed = true
  elseif drag.floor and to_inv and not to_inv.worn then
    -- Drag from floor to inventory; pick up.
    self.actor:pickupFloor(drag.item_idx, true)
    self.actor.changed = true
    changed = true
  elseif from_inv and from_inv.worn and to_inv and not to_inv.worn then
    -- Drag from equip to inventory; unwield.
    self.actor:doTakeoff(drag.inven, drag.item_idx, drag.object)
    changed = true
  elseif to_inv and to_inv.worn and (drag.floor or (from_inv and not from_inv.worn)) then
    -- Drag from floor or inventory to equip; wield.
    if self.actor:canWearObject(drag.object, to_inv.name) then
      self.actor:doWear(drag.inven, drag.floor, drag.item_idx, drag.object, self.actor, item.inven, item.item)
      changed = true
    end
  end

  if changed then
    self:generateList()
    game.mouse:usedDrag()
  end
end

function _M:inventoryLiveUpdate()
  self:generateList()
end
