-- $Id: StoreItemDialog.lua 2379 2016-05-01 23:30:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require "engine.class"
local Dialog = require "engine.ui.Dialog"
local GetQuantity = require 'engine.dialogs.GetQuantity'
local List = require "engine.ui.List"

module(..., package.seeall, class.inherit(Dialog))

function _M:init(center_mouse, actor, store, object, item, store_inven, dlg)
  self.actor = actor
  self.store = store
  self.object = object
  self.inven = store_inven
  self.item = item
  self.store_dialog = dlg

  self:generateList()
  local name = object:getName()
  Dialog.init(self, name, 1, 1)

  local w = self.font_bold:size(name)
  local list = List.new {
    width = math.max(w, self.max_w) + 10,
    nb_items = #self.list,
    list = self.list,
    fct = function(item) self:use(item) end,
  }

  local center_cb = function(w, h)
    if center_mouse then
      local mx, my = core.mouse.get()
      self.force_x = mx - w / 2
      self.force_y = my - (self.h - self.ih + list.fh / 3)
    end
  end
  self:loadUI { { left=0, top=0, ui=list } }
  self:setupUI(true, true, center_cb)

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end
  }
end

function _M:use(item)
  if not item then return end
  game:unregisterDialog(self)

  if item.action == 'buy' then
    self.store_dialog.action('buy', self.object, self.item)
  elseif item.action == 'browse' then
    local UseTalents = require('mod.dialogs.UseTalents')
    local d = UseTalents.new(self.actor, self.object, {no_use=true, no_bind=true})
    game:registerDialog(d)
  elseif item.action == 'debug-inspect' then
    local DebugConsole = require"engine.DebugConsole"
    local d = DebugConsole.new()
    game:registerDialog(d)
    DebugConsole.line = "=__uids["..self.object.uid.."]"
    DebugConsole.line_pos = #DebugConsole.line
    d.changed = true
  end
end

function _M:generateList()
  local list = {}

  if util.getval(self.store_dialog.allow_buy, self.object, self.item) then
    list[#list+1] = { name=self.store:buyActionName(), action='buy' }
  end
  if spell.item_spells(self.object, self.actor, true) then
    list[#list+1] = { name='Browse spells', action='browse' }
  end
  if config.settings.cheat then
    list[#list+1] = { name='Lua inspect', action='debug-inspect', color=colors.simple(colors.LIGHT_BLUE) }
  end

  self.max_w = 0
  self.max_h = 0
  for i, v in ipairs(list) do
    local w, h = self.font:size(v.name)
    self.max_w = math.max(self.max_w, w)
    self.max_h = self.max_h + self.font_h
  end

  self.list = list
end
