-- $Id: TacticExplorDialog.lua 2379 2016-05-01 23:30:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
--local ListColumns = require 'engine.ui.ListColumns'
local TweakedTreeList = require 'mod.dialogs.ui.TweakedTreeList'
local Textzone = require 'engine.ui.Textzone'
local Separator = require 'engine.ui.Separator'

module(..., package.seeall, class.inherit(Dialog))

local settings
settings = {
  tactic = {
    { name='coward', hit=-10, dam=-10, ac=15, stlth=3, disarm=15, save=14 },
    { name='meek', hit=-8, dam=-8, ac=10, stlth=2, disarm=9, save=9 },
    { name='wary', hit=-4, dam=-4, ac=5, stlth=1, disarm=5, save=5 },
    { name='careful', hit=-2, dam=-2, ac=2, stlth=1, disarm=2, save=2 },
    { name='normal', hit=0, dam=0, ac=0, stlth=0, disarm=0, save=0 },
    { name='confident', hit=2, dam=2, ac=-2, stlth=-1, disarm=-2, save=-3 },
    { name='aggressive', hit=4, dam=4, ac=-5, stlth=-2, disarm=-5, save=-7 },
    { name='furious', hit=6, dam=6, ac=-10, stlth=-3, disarm=-11, save=-12 },
    { name='berserker', hit=8, dam=12, ac=-25, stlth=-5, disarm=-18, save=-18 },
    fmt = [[Tactic: #00FF00#%s#LAST#
Effects:
  #00FF00#%+d#LAST# to-hit
  #00FF00#%+d#LAST# to-damage
  #00FF00#%+d#LAST# armor
  #00FF00#%+d#LAST# stealth
  #00FF00#%+d#LAST# disarming
  #00FF00#%+d#LAST# saving throw]],
    desc = function(item)
      return string.format(settings.tactic.fmt, item.name, item.hit, item.dam, item.ac, item.stlth, item.disarm, item.save)
    end,
  },
  explor = {
    { name='slug-like', speed=-10, srch=17, stlth=4, percep=20 },
    { name='very slow', speed=-8, srch=12, stlth=4, percep=16 },
    { name='slow', speed=-6, srch=8, stlth=3, percep=10 },
    { name='leisurely', speed=-3, srch=4, stlth=2, percep=6 },
    { name='normal', speed=0, srch=0, stlth=0, percep=0 },
    { name='brisk', speed=1, srch=-4, stlth=-1, percep=-4 },
    { name='fast', speed=2, srch=-6, stlth=-4, percep=-8 },
    { name='very fast', speed=3, srch=-10, stlth=-7, percep=-14 },
    { name='running', speed=4, srch=-16, stlth=-10, percep=-20 },
    fmt = [[Explor: #00FF00#%s#LAST#
Effects:
  #00FF00#%+d#LAST# speed
  #00FF00#%+d#LAST# searching
  #00FF00#%+d#LAST# stealth
  #00FF00#%+d#LAST# perception]],
    desc = function(item)
      return string.format(settings.explor.fmt, item.name, item.speed, item.srch, item.stlth, item.percep)
    end,
  }
}

function _M:init(actor)
  Dialog.init(self, 'Tactic/Explor', 600, 400)
  self.actor = actor

  self.hs = Separator.new { dir='horizontal', size=self.ih }

  self.c_desc = Textzone.new {
    width = math.floor(self.iw/2) - 10 - self.hs.w,
    height = self.ih,
    scrollbar = true,
    no_color_bleed = true,
    text = '',
  }
  self:generateList()
  --self.c_list = ListColumns.new {
  self.c_list = TweakedTreeList.new {
    simple_list = true,
    width = math.floor(self.iw/2) - 10 - self.hs.w,
    height = self.ih,
    --list = self.working_settings,
    tree = self.working_settings,
    columns = {
      { name='Setting', width=60, display_prop='name' },
      { name='Value', width=40, display_prop='display_val' },
    },
    fct = function(item, _, _)
      self:adjSetting(item, v == 'left' and 1 or -1)
    end,
    select = function(item, sel)
      self.sel_setting = item
      self:populateDesc(item)
    end,
  }
  self.c_list.key:removeBind('ACCEPT')

  self:loadUI {
    { left=0, top=0, ui=self.c_list },
    { left=self.c_list, top=0, ui=self.hs },
    { right=0, top=0, ui=self.c_desc },
  }
  self:setFocus(self.c_list)
  self:setupUI()
  self.key:addBinds {
    ACCEPT = function() game:unregisterDialog(self) end,
    EXIT = function() game:unregisterDialog(self) end,
    MOVE_LEFT = function() self:adjSetting(self.sel_setting, -1) end,
    MOVE_RIGHT = function() self:adjSetting(self.sel_setting, 1) end,
  }
end

function _M:generateList()
  local tval = self.actor.tactic or 5
  local tdisp = settings.tactic[tval].name
  local eval = self.actor.explor or 5
  local edisp = settings.explor[eval].name
  local list = {
    { id='tactic', name='Tactic', val=tval, display_val=tdisp },
    { id='explor', name='Explor', val=eval, display_val=edisp },
  }

  self.working_settings = list
  if self.c_list then
    --self.c_list.list = self.working_settings
    self.c_list.tree = self.working_settings
    self.c_list:generate()
  end
end

function _M:adjSetting(item, delta)
  if not item then return end
  item.val = util.boundWrap(item.val + delta, 1, #settings[item.id])
  item.display_val = settings[item.id][item.val].name
  self.c_list:drawItem(item)
  self:populateDesc(item)
  self.actor[item.id] = item.val
  self.actor:recalcEverything()
  self.actor.changed = true
end

function _M:populateDesc(item)
  self.c_desc.text = settings[item.id].desc(settings[item.id][item.val])
  self.c_desc:generate()
end

function _M:apply(actor)
  actor.tactic = actor.tactic or 5
  actor.explor = actor.explor or 5
  local tactic = settings.tactic[actor.tactic]
  local explor = settings.explor[actor.explor]

  actor:adjustDis('to_h', tactic.hit, true, tactic.name .. ' tactic')
  actor:adjustDis('to_d', tactic.dam, true)
  actor:adjustDis('to_a', tactic.ac, true)
  actor.skill_stealth = actor.skill_stealth + tactic.stlth
  actor.skill_disarm = actor.skill_disarm + tactic.disarm
  actor.saving_throw = actor.saving_throw + tactic.save

  actor.speed = actor.speed + explor.speed
  actor.skill_searching = actor.skill_searching + explor.srch
  actor.skill_stealth  = actor.skill_stealth + explor.stlth
  actor.skill_perception = actor.skill_perception + explor.percep
end

function _M:getCurDesc(actor, which)
  actor.tactic = actor.tactic or 5
  actor.explor = actor.explor or 5
  if which then return settings[which][actor[which]].name end
  return settings.tactic[actor.tactic].name .. '/' .. settings.explor[actor.explor].name
end
