-- $Id: FindUniqueDialog.lua 1993 2015-12-20 04:15:40Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local List = require 'engine.ui.List'

module(..., package.seeall, class.inherit(Dialog))

function _M:init()
  self:generateList()
  Dialog.init(self, 'Find Unique', 1, 1)

  self.c_list = List.new {
    width = 400,
    height = 500,
    scrollbar = true,
    list = self.list,
    fct = function(item) self:use(item) end,
  }

  self:loadUI {
    { left=0, top=0, ui=self.c_list },
  }
  self:setupUI(true, true)

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end,
  }
end

function _M:on_register()
  game:onTickEnd(function() self.key:unicodeInput(true) end)
end

function _M:use(item)
  if not item or not item.actor then return end
  game:unregisterDialog(self)
  game.level.map:moveViewSurround(item.actor.x, item.actor.y, 1000, 1000)
  -- Slight Hack(TM):  Our target almost certainly isn't in LOS, so we need
  -- to set the 'always_visible' flag on this particle.
  local e = require('engine.Particles').new('notice_grid', 1)
  e.x, e.y, e.always_visible = item.actor.x, item.actor.y, true
  game.level.map:addParticleEmitter(e)
end

function _M:generateList()
  local list = {}

  for _, e in pairs(game.level.entities) do
    if e:getEntityKind() == 'actor' and e.unique and not e.player then
      local name = ('%s @ %d,%d [UID: %d]'):format(e:getName(), e.x, e.y, e.uid)
      list[#list+1] = { name=name, actor=e }
    end
  end

  table.sort(list, function(a, b) return a.name < b.name end)

  local chars = {}
  for i, _ in ipairs(list) do chars[self:makeKeyChar(i)] = i end
  list.chars = chars

  self.list = list
end
