-- $Id: load.lua 2558 2016-10-04 00:27:43Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright (C) 2012 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbbunnies.org

-- This file loads the game module, and loads data
require 'mod.class.Map'
-- Spectacularly Evil and Annoying Hack(TM):  This modifies
-- GameEnergyBased:tick() in place.
local GameEnergyBasedEvilHack = require 'mod.class.GameEnergyBasedEvilHack'
local KeyBind = require "engine.KeyBind"
local DamageType = require "engine.DamageType"
local AttackType = require 'mod.class.AttackType'
local ObjectData = require 'mod.class.interface.ObjectData'
local Ego = require 'mod.class.Ego'
local ActorResource = require "engine.interface.ActorResource"
local ActorTalents = require "engine.interface.ActorTalents"
local ActorInventory = require 'engine.interface.ActorInventory'
local PlayerSkills = require 'mod.class.interface.PlayerSkills'
local PlayerCombat = require 'mod.class.interface.PlayerCombat'
local ActorAI = require "engine.interface.ActorAI"
local ActorTemporaryEffects = require "engine.interface.ActorTemporaryEffects"
local NPCFields = require 'mod.class.interface.NPCFields'
local NPCEgo = require 'mod.class.NPCEgo'
local Birther = require "engine.Birther"
local GameOptions = require 'mod.dialogs.GameOptions'
local Store = require 'mod.class.Store'
local StoreOwner = require 'mod.class.StoreOwner'
local Home = require 'mod.class.Home'
local MathomHouse = require 'mod.class.MathomHouse'
local T2DungeonRooms = require 'mod.class.generator.map.T2DungeonRooms'
local WorldAchievements = require "engine.interface.WorldAchievements"
local HotkeysIcons = require 'mod.class.HotkeysIcons'

-- Initialize settings.
GameOptions:loadDefinition('/data/options.lua')
GameOptions:setupOptions()

-- How kosher is it for modules to use engine hooks? :-}
-- Apply visuals mods from user prefs to any entity list we load.
class:bindHook('Entity:loadList', function(self, data)
  if #data.res == 0 then return end
  local key = ('ascii_%s_symbols'):format(data.res[1]:getEntityKind())
  local mods = config.settings.tome2[key]
  if mods then
    local VisualsDialog = require 'mod.dialogs.VisualsDialog'
    VisualsDialog:updateEntityList(data.res, mods)
  end
end)

-- The 16 predefined colors from the original *_info.txt files, for easy
-- translation.
defineColor('C_d', 0x00, 0x00, 0x00)
defineColor('C_w', 0xFF, 0xFF, 0xFF)
defineColor('C_s', 0x80, 0x80, 0x80)
defineColor('C_o', 0xFF, 0x80, 0x00)
defineColor('C_r', 0xC0, 0x00, 0x00)
defineColor('C_g', 0x00, 0x80, 0x00)
defineColor('C_b', 0x00, 0x00, 0xFF)
defineColor('C_u', 0x80, 0x40, 0x00)
defineColor('C_D', 0x40, 0x40, 0x40)
defineColor('C_W', 0xC0, 0xC0, 0xC0)
defineColor('C_v', 0xFF, 0x00, 0xFF)
defineColor('C_y', 0xFF, 0xFF, 0x00)
defineColor('C_R', 0xFF, 0x00, 0x00)
defineColor('C_G', 0x00, 0xFF, 0x00)
defineColor('C_B', 0x00, 0xFF, 0xFF)
defineColor('C_U', 0xC0, 0x80, 0x40)
-- And since the engine's TString parsing doesn't recognize eg. #C_l# as a
-- color if 'l' is lower-case, we define aliases of the form C__L for the
-- lower-case colors.
defineColor('C__D', 0x00, 0x00, 0x00)
defineColor('C__W', 0xFF, 0xFF, 0xFF)
defineColor('C__S', 0x80, 0x80, 0x80)
defineColor('C__O', 0xFF, 0x80, 0x00)
defineColor('C__R', 0xC0, 0x00, 0x00)
defineColor('C__G', 0x00, 0x80, 0x00)
defineColor('C__B', 0x00, 0x00, 0xFF)
defineColor('C__U', 0x80, 0x40, 0x00)
defineColor('C__V', 0xFF, 0x00, 0xFF)
defineColor('C__Y', 0xFF, 0xFF, 0x00)
-- TODO Implement multi-color handling.
defineColor('C_multi', 0xFF, 0x00, 0xFF)

-- Useful keybinds
KeyBind:load("move,hotkeys,inventory,actions,interface,tome2")

-- Damage types
DamageType:loadDefinition("/data/damage_types.lua")
AttackType:loadDefinition('/data/attack_types.lua')

dofile('/mod/util.lua')
dofile('/mod/spell-util.lua')
dofile('/mod/ui-util.lua')
dofile('/mod/obj-util.lua')
dofile('/mod/trap-util.lua')
dofile('/mod/resolvers.lua')

-- Achievements
WorldAchievements:loadDefinition("/data/achievements/")

-- Talents
ActorTalents:loadDefinition('/data/talents/talents.lua')

-- Skills and abilities
PlayerSkills:loadDefinition('/data/skills/skills.lua')

-- Melee styles
PlayerCombat:loadMeleeStyles('/data/melee-styles.lua')

-- Timed Effects
ActorTemporaryEffects:loadDefinition("/data/timed_effects.lua")

-- Actor resources
ActorResource:defineResource('Mana', 'mana', nil, 'mana_regen', 'Mental power, used to cast spells or perform some innate abilities.')
ActorResource:defineResource('Piety', 'piety', nil, 'piety_regen', 'Spiritual power, used to cast spells granted to you by your god.', -300000, 300000)
ActorResource:defineResource('Food', 'food', nil, 'food_regen', [[A measure of how much you've eaten, and thus how long you can go before starving.]], 0, 20000)

-- Actor inventory
ActorInventory:defineInventory('MAIN_HAND', 'Main hand', true, 'Weapon')
ActorInventory:defineInventory('OFF_HAND', 'Off hand', true, 'Shield')
ActorInventory:defineInventory('BODY', 'Armor', true, 'Body armor')
ActorInventory:defineInventory('CLOAK', 'Cloak', true, 'Cloak')
ActorInventory:defineInventory('LAUNCHER', 'Launcher', true, 'Bow, crossbow or sling')
ActorInventory:defineInventory('QUIVER', 'Quiver', true, 'Arrows, crossbow bolts or sling shots')
ActorInventory:defineInventory('GLOVES', 'Hands', true, 'Gloves or gauntlets')
ActorInventory:defineInventory('BOOTS', 'Feet', true, 'Boots')
ActorInventory:defineInventory('HELM', 'Head', true, 'Helmets or other headwear')
ActorInventory:defineInventory('RING', 'Fingers', true, 'Rings')
ActorInventory:defineInventory('AMULET', 'Neck', true, 'Amulet')
ActorInventory:defineInventory('LITE', 'Light', true, 'Torches or lantern')
ActorInventory:defineInventory('TOOL', 'Tool', true, 'Shovel or other tool')
ActorInventory:defineInventory('SYMBIOTE', 'Symbiote', true, 'A hypnotized non-moving creature worn on your body.  Requires Symbiosis skill.', nil, { hide=function(a) return not a.getSkillLevel or a:getSkillLevel('SYMBIOSIS') <= 0 end })

-- Factions
dofile('/data/factions.lua')

-- Actor AIs
ActorAI:loadDefinition("/engine/ai/")
ActorAI:loadDefinition('/mod/class/ai/')

-- Birther descriptor
Birther:loadDefinition("/data/birth/descriptors.lua")

-- Stores
StoreOwner:loadOwners('/data/general/stores/owners.lua')
Store:loadStores('/data/general/stores/stores.lua')
Home:loadHomes('/data/general/stores/homes.lua')
MathomHouse:loadMuseums('/data/general/stores/mathom-houses.lua')

-- Set chat dialogs to show portraits.
local ChatDialog = require 'engine.dialogs.Chat'
ChatDialog.show_portraits = true

-- Object-relataed stuff.
ObjectData:loadFlavors('/data/object_flavors.lua')
ObjectData:loadJunkarts('/data/junkart_data.lua')
ObjectData:loadMimicryForms('/data/mimicry_forms.lua')
ObjectData:loadFieldDescriptions('/data/object_fields.lua')
ObjectData:loadFlagCallbacks('/data/object_flag_callbacks.lua')

-- Object egos
Ego:loadEgos('/data/general/objects/egos.lua')

-- NPC-related stuff
NPCFields:loadFieldDescriptions('/data/actor_fields.lua')
NPCEgo:loadEgos('/data/general/npcs/egos.lua')

-- Room definitions
T2DungeonRooms:loadDefinition('/data/rooms/rooms.lua')

-- Hotkey icon types
HotkeysIcons:loadDefinition('/data/hotkey-icons.lua')

-- Walk the /data/quests directory (and corresponding directories of any
-- addons, if I've done this right) and call the add_hooks() method for
-- each quest that defines one.
local function load_quest_hooks(basedir)
  for _, file in ipairs(fs.list(basedir)) do
    if file:find('.lua$') then
      local f, err = loadfile(basedir .. file)
      if not f and err then error(err) end
      local ret = {}
      setfenv(f, setmetatable(ret, {__index=_G}))
      f()
      if ret.add_hooks then ret.add_hooks() end
    end
  end
end
load_quest_hooks('/data/quests/')
for _, dir in ipairs(fs.list('/')) do
  local q_dir = '/'..dir..'/quests/'
  local _, _, addon = dir:find('^data%-(.+)$')
  if addon and fs.exists(q_dir) then load_quest_hooks(q_dir) end
end

class:triggerHook{ 'T2:load' }

return { require 'mod.class.Game', require 'mod.class.World' }
